﻿/**
  ******************************************************************************
  * @file    Constants.cs
  * @author  Happiest Minds Firmware Team
  * @version v1.0
  * @date    13-December-2016
  * @brief   This file contains defined constants
  ******************************************************************************
  */

using System;

namespace mPCIe_DIO_24s
{
    static class DeviceInfo
    {
        public const string SOFTWARE_VERSION    = "v1.2";
        public const string DEVICE_NAME         = "ACCES mPCIe Card";
        public const UInt32 DEVICE_ID           = 0x0E57;
    }

    /// <summary>
    /// Enumerations for return status
    /// </summary>
    public enum eStatus
    {
        eSuccess = 0,
        eFailure = -1,
    };

    /// <summary>
    /// Enumeration of Available ports
    /// </summary>
    public enum ePortName
    {
        ePortA = 0,
        ePortB,
        ePortCL,
        ePortCH,
        ePortUnknown
    };

    /// <summary>
    /// /// Enumeration of Available pins
    /// </summary>
    public enum ePinNum
    {
        ePin0 = 0,
        ePin1,
        ePin2,
        ePin3,
        ePin4,
        ePin5,
        ePin6,
        ePin7,
        ePinUnknown
    };

    /// <summary>
    /// Enumeration for Direction
    /// </summary>
    public enum eDirection
    {
        eInput = 0,
        eOutput,
        eUnknown
    };

    /// <summary>
    /// Enumeration for Supported Alternate functions
    /// </summary>
    public enum eAltFunction
    {
        // Input Alternate functions
        eGenIn = 0,
        ePulseDetect,
        eLtoHDetect,
        eCntPulse,
        eCntQuadPulse,
        eMeasPulseHigh,
        eMeasFreq,
        eMeasDutyCycle,
        // Output Alternate functions
        eGenOut,
        eGenPulse,
        eGenPulseTrain,
        eGenDutyCycle,
        eGenQuad,
        eNone
    }

    /// <summary>
    /// Supported configuration parameters
    /// </summary>    
    public enum eConfigParam
    {
        eGenOutput = 0,      
        eGenInput,    
        eRisingEdge,
        eFallingEdge,
        eLowPulse,
        eHighPulse, 
        eFreqMeasure,
        ePulseDirHigh,
        ePulseDirLow, 
        eDirForward,
        eDirReverse,      
        eNone
    };

    /// <summary>
    /// Structure Detect Pulse Configuration parameter
    /// </summary>
    public struct DetectPulseConfig
    {
        public eConfigParam TriggerType;
        public bool EnableIrq;
    }    

    /// <summary>
    /// Structure for Event counter Configuration parameter
    /// </summary>
    public struct EventCounterConfig
    {
        public eConfigParam TriggerType;
        public UInt16 CounterThreshold;        
    }

    /// <summary>
    /// Structure Pulse Train Configuration parameter
    /// </summary>
    public struct PulseTrainConfig
    {
        public eConfigParam PulseDir;
        public float PulseLengthMilliSec;
        public UInt16 PulseCount;
        public float TimeBetweenPulsesMilliSec;
    }

    /// <summary>
    /// Structure Duty cycle Configuration parameter
    /// </summary>
    public struct DutyCycleConfig
    {
        public float TimePeriodMilliSec;
        public int DutyCycle;
    }

    /// <summary>
    /// 
    /// </summary>
    public struct MotorOpConfig
    {     
        public eConfigParam Direction;
        public float BaseFreqKhz;
        public UInt32 OffsetCount;
    }

    /// <summary>
    /// 
    /// </summary>
    public struct QuadratureIpConfig
    {
        public bool IsResetConfig;
        public ePortName QdcResetPinName;
        public ePinNum QdcResetPinNumber;
        public eConfigParam QdcResetTriggerType;
    }

    /// <summary>
    /// Structure for all configuration parameters
    /// </summary>
    public struct UpdateConfigParam
    {
        public ePortName PortName;
        public ePinNum PinNumber;
        public eAltFunction AltFun;
        public bool EnableDebounce;        
        public bool ResetCounter;
        public bool OutputState;        
        public EventCounterConfig EventCounterConfig;
        public DetectPulseConfig PulseDetectConfig;
        public QuadratureIpConfig QdcConfig;        
        public PulseTrainConfig PulseTrainConfig;
        public DutyCycleConfig DutyCycleConfig;
        public MotorOpConfig McsConfig;
    }

    /// <summary>
    /// Register values
    /// </summary>
    static class RegData
    {
        public const UInt32 AOP_BIT             = 0x04;
        public const UInt32 IO_ENABLE           = 0x80;
        public const UInt32 PORTA_DIR_IN        = 0x10;
        public const UInt32 PORTB_DIR_IN        = 0x02;
        public const UInt32 PORTCL_DIR_IN       = 0x01;
        public const UInt32 PORTCH_DIR_IN       = 0x08;
        // Input Alternate Functions
        public const byte ALT_FUN_H2L           = 0x01;
        public const byte ALT_FUN_L2H           = 0x02;
        public const byte ALT_FUN_LOW_Pulse     = 0x04;
        public const byte ALT_FUN_HIGH_Pulse    = 0x08;
        public const byte ALT_FUN_IPHL          = 0x10;
        public const byte ALT_FUN_IPWM          = 0x20;
        public const byte ALT_FUN_IFQM          = 0x40;
        public const byte ALT_FUN_IDBE          = 0x80;
        // Output Alternate Functions
        public const UInt32 ALT_FUN_OPTG        = 0x10000;
        public const UInt32 ALT_FUN_OPWM        = 0x20000;
        public const UInt32 IRQ_EN              = 0x40000;
        public const UInt32 OPTG_DIR            = 0x80000;
        //Motor Control Output
        public const UInt32 MCS_SEL             = 0x10;
        public const UInt32 MCS_DIR             = 0x40;
        public const UInt32 MCS_CTRL            = 0x20;
        // Quadrature Counter
        public const UInt32 QDC_SEL             = 0x20;
        public const UInt32 QDC_RST             = 0x10;
        public const UInt32 QDC_ENABLE_IDX      = 0x40;

        public const UInt32 ECOFF_BIT           = 0x8000;
        public const UInt32 IDBE_BIT            = 0x80;

        // Rest bits
        public const UInt32 GRSTC_BIT           = 0x01;
        public const UInt32 GRST_BIT            = 0x02;

        // Global Interrupt Enable/Disable Register Address
        public const byte GLOBAL_INT_ENABLE       = 0xFF;
        public const byte GLOBAL_INT_STATUS_CLEAR = 0xFF;
    }
    /// <summary>
    /// Register offset base address
    /// </summary>
    static class RegisterOffsetBase
    {
        public const UInt16 PORTA_REG = 0x0000;
        public const UInt16 PORTB_REG = 0x0001;
        public const UInt16 PORTC_REG = 0x0002;
        public const UInt16 PORT_DIR_REG = 0x0003;
        public const UInt16 IO_ENABLE_IOREG = 0x0008;

        // Global Interrupt Enable/Disable Register Address
        public const UInt16 GLOBAL_INT_ENABLE_REG = 0x0028;
        public const UInt16 GLOBAL_INT_STATUS_CLEAR_REG = 0x0029;

        public const UInt16 INT_ENABLE_REG = 0x0030; // 0x0034;
        public const UInt16 INT_STATUS_REG = 0x0040; // 38
        public const UInt16 CTRL_REG = 0x003C; // 3C

        // Quadrature Counter Registers
        public const UInt16 QDC_CONFIG_REG = 0x60; // 40
        public const UInt16 QDC_COUNTER_REG = 0x64; // 44

        // Motor Control Registers
        public const UInt16 MCS_CONFIG_REG = 0x70; // 60
        public const UInt16 MCS_FREQ_REG = 0x74; // 64
        public const UInt16 MCS_COUNT_REG = 0x78; // 68

        // resets register
        public const UInt16 RESETS_REG = 0xFC; // added after release 1.3

        // Port A newly added register offset address
        public const UInt16 PA0_OFFSET = 0x0100;      // Line 1
        public const UInt16 PA1_OFFSET = 0x0200;
        public const UInt16 PA2_OFFSET = 0x0300;
        public const UInt16 PA3_OFFSET = 0x0400;
        public const UInt16 PA4_OFFSET = 0x0500;
        public const UInt16 PA5_OFFSET = 0x0600;
        public const UInt16 PA6_OFFSET = 0x0700;
        public const UInt16 PA7_OFFSET = 0x0800;

        // Port B newly added register offset address
        public const UInt16 PB0_OFFSET = 0x0900;      // Line 9
        public const UInt16 PB1_OFFSET = 0x0A00;
        public const UInt16 PB2_OFFSET = 0x0B00;
        public const UInt16 PB3_OFFSET = 0x0C00;
        public const UInt16 PB4_OFFSET = 0x0D00;
        public const UInt16 PB5_OFFSET = 0x0E00;
        public const UInt16 PB6_OFFSET = 0x0F00;
        public const UInt16 PB7_OFFSET = 0x1000;

        // Port C newly added register offset address
        public const UInt16 PC0_OFFSET = 0x1100;      // Line 17
        public const UInt16 PC1_OFFSET = 0x1200;
        public const UInt16 PC2_OFFSET = 0x1300;
        public const UInt16 PC3_OFFSET = 0x1400;
        public const UInt16 PC4_OFFSET = 0x1500;
        public const UInt16 PC5_OFFSET = 0x1600;
        public const UInt16 PC6_OFFSET = 0x1700;
        public const UInt16 PC7_OFFSET = 0x1800;      // Line 24

        // Invalid Address
        public const UInt32 NOT_USED = 0xAA55;
    }

    static class BitRegisterOffset
    {
        public const UInt16 FEATURE_REG_OFFSET = 0x00;
        public const UInt16 EVENT_REG_OFFSET = 0x04;
        public const UInt16 COUNTER_8BIT_EVENTPULSEOUT = 0x08;
        public const UInt16 EVENT_THRESHOLD_REG_OFFSET = 0x0C;
        public const UInt16 PULSE_REG_OFFSET = 0x10;
        public const UInt16 FREQUENCY_COUNTER_REG_OFFSET = 0x1C;
        public const UInt16 PULSE_LO_REG_OFFSET = 0x20;
        public const UInt16 PULSE_HI_REG_OFFSET = 0x24;
        public const UInt16 PWM_LO_MEASURED_REG_OFFSET = 0x30;
        public const UInt16 PWM_HI_MEASURED_REG_OFFSET = 0x34;
    }
}

/***************************************************************************************************************/
/* End of File                                                                                                 */
/***************************************************************************************************************/
