﻿/**
  ******************************************************************************
  * @file    EventCntConfigWinform.cs
  * @author  Happiest Minds Firmware Team
  * @version v1.0
  * @date    13-December-2016
  * @brief   This file handles the Event counter configuration dialog box
  ******************************************************************************
  */
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace mPCIe_DIO_24s
{
    public partial class EventCntConfigWinform : Form
    {
        /*************************************************************************************************************
         * 
         * Public Functions
         * 
         *************************************************************************************************************/
        /// <summary>
        /// Initialize the EventCntConfigWinform components
        /// </summary>
        public EventCntConfigWinform()
        {
            InitializeComponent();
        }

        /*************************************************************************************************************
         * 
         * Private Functions
         * 
         *************************************************************************************************************/
        /// <summary>
        /// Initialize the Event Counter dialog window and loads the previous configuration values
        /// </summary>
        /// <param name="sender">The object which has raised the event</param>
        /// <param name="e">Additional information about the event</param>
        private void EventCntConfigWinform_Load(object sender, EventArgs e)
        {
            // Add the supported cout type string
            string[] supportedCountType = new string[] { "Rising Edge", "Falling Edge", "High Pulse", "Low Pulse" };
            cbCountType.Items.AddRange(supportedCountType);

            // Select Rising edge as default configuration 
            cbCountType.SelectedIndex = 0;

            // Load the previous configured values
            // Get the port name and pin number
            Tuple<ePortName, ePinNum> pin;
            pin = ucPinConfig.GetLastUpdatedPortPinName();

            // If it is already configured, load the previous configured values
            if (Main_Form.ConfigurationParam[(int)pin.Item1, (int)pin.Item2].EventCounterConfig.CounterThreshold != 0)
            {
                // Update the counter Threshold value
                tbCountThreshold.Text = Main_Form.ConfigurationParam[(int)pin.Item1, (int)pin.Item2].EventCounterConfig.CounterThreshold.ToString();

                // Update the count type selection 
                if (Main_Form.ConfigurationParam[(int)pin.Item1, (int)pin.Item2].EventCounterConfig.TriggerType == eConfigParam.eRisingEdge)
                    cbCountType.SelectedIndex = 0;
                else if (Main_Form.ConfigurationParam[(int)pin.Item1, (int)pin.Item2].EventCounterConfig.TriggerType == eConfigParam.eFallingEdge)
                    cbCountType.SelectedIndex = 1;
                else if (Main_Form.ConfigurationParam[(int)pin.Item1, (int)pin.Item2].EventCounterConfig.TriggerType == eConfigParam.eHighPulse)
                    cbCountType.SelectedIndex = 2;
                else if (Main_Form.ConfigurationParam[(int)pin.Item1, (int)pin.Item2].EventCounterConfig.TriggerType == eConfigParam.eLowPulse)
                    cbCountType.SelectedIndex = 3;
            }
        }

        /// <summary>
        /// Allow only Numeric Values in the text box. Block all other characters
        /// </summary>
        /// <param name="sender">The object which has raised the event</param>
        /// <param name="e">Additional information about the event</param>
        private void tbCountThreshold_KeyPress(object sender, KeyPressEventArgs e)
        {
            e.Handled = !char.IsDigit(e.KeyChar) && !char.IsControl(e.KeyChar);
        }

        /// <summary>
        /// OK Button click event handler. Validate and update the configuration
        /// </summary>
        /// <param name="sender">The object which has raised the event</param>
        /// <param name="e">Additional information about the event</param>
        private void btOk_Click(object sender, EventArgs e)
        {
            eConfigParam CountType = eConfigParam.eRisingEdge;
            UInt16 threshold = 0;

            if (tbCountThreshold.Text == "")
            {
                MessageBox.Show("Please enter valid Threshold Count value");
            }
            else
            {
                try
                {
                    // Get the Threshold value from text box
                    threshold = UInt16.Parse(tbCountThreshold.Text);
                }
                catch
                {
                    MessageBox.Show("Entered Counter Threshold value is invalid. May be too long");
                    return;
                }

                /* Make sure that entered value is non zero */
                if (threshold == 0)
                {
                    MessageBox.Show("Counter Threshold value is invalid. Value should be non-zero");
                    return;
                }

                // Check if threshold value is out of range
                if (threshold <= byte.MaxValue)
                {
                    // Get the Count type value from combo box
                    if (cbCountType.SelectedIndex == 0)
                        CountType = eConfigParam.eRisingEdge;
                    else if (cbCountType.SelectedIndex == 1)
                        CountType = eConfigParam.eFallingEdge;
                    else if (cbCountType.SelectedIndex == 2)
                        CountType = eConfigParam.eHighPulse;
                    else if (cbCountType.SelectedIndex == 3)
                        CountType = eConfigParam.eLowPulse;

                    // Update the configured values
                    ucPinConfig.EventCounterConfigUpdate(CountType, threshold);
                    //Close the pop-up window
                    this.Close();                    
                }
                else
                {
                    MessageBox.Show("Please enter Threshold Count value below " + byte.MaxValue);
                }
            }                                    
        }

        /// <summary>
        /// Event Count Configuration window closed event Handler Dispose the form
        /// </summary>
        /// <param name="sender">The object which has raised the event</param>
        /// <param name="e">Additional information about the event</param>
        private void EventCntConfigWinform_FormClosed(object sender, FormClosedEventArgs e)
        {
            // Dispose of unmanaged resources.
            Dispose(true);
            GC.SuppressFinalize(this);
        }
    }
}
/***************************************************************************************************************/
/* End of File                                                                                                 */
/***************************************************************************************************************/