﻿/**
  ******************************************************************************
  * @file    MainForm.cs
  * @author  Happiest Minds Firmware Team
  * @version v1.0
  * @date    13-December-2016
  * @brief   This file does all read write operation for mPCIe card
  ******************************************************************************
  */
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using mPCIe_DIO_24s;
using System.Threading;
using System.IO;
using AIOWDMNet;  // the namespace exposes the AIOWDM Class interface 

namespace mPCIe_DIO_24s
{
    public partial class Main_Form : Form
    {
        /*************************************************************************************************************
         * 
         * Public Variable declarations
         * 
         *************************************************************************************************************/        
        public static UpdateConfigParam[,] ConfigurationParam = new UpdateConfigParam[4, 8];    // Two Dimensional array holds all the configuration parameters    
        public static bool IsMcsUsed = false;   // Flag for is MCS configured
        public static bool IsQdcUsed = false;   // Flag for is QDC configured  
        public static Int32 cardNumber = 0;     // The index of the one card we will be using 0        

        /*************************************************************************************************************
         * 
         * Private Defined Constants
         * 
         *************************************************************************************************************/
        private const double TIME_ONE_LSB = 8e-9;        // 1 LSB = 8 nano Second

        /*************************************************************************************************************
         * 
         * Private variable declaration
         * 
         *************************************************************************************************************/
        private static AIOWDMLog logUI; // reference to the logging form


        // Array for Pin Base Address for all 24 pins
        private UInt32[,] pinConfigRegBaseAddress = new UInt32[4, 8] {
            { RegisterOffsetBase.PA0_OFFSET, RegisterOffsetBase.PA1_OFFSET, RegisterOffsetBase.PA2_OFFSET, RegisterOffsetBase.PA3_OFFSET, RegisterOffsetBase.PA4_OFFSET, RegisterOffsetBase.PA5_OFFSET, RegisterOffsetBase.PA6_OFFSET, RegisterOffsetBase.PA7_OFFSET} ,
            { RegisterOffsetBase.PB0_OFFSET, RegisterOffsetBase.PB1_OFFSET, RegisterOffsetBase.PB2_OFFSET, RegisterOffsetBase.PB3_OFFSET, RegisterOffsetBase.PB4_OFFSET, RegisterOffsetBase.PB5_OFFSET, RegisterOffsetBase.PB6_OFFSET, RegisterOffsetBase.PB7_OFFSET} ,
            { RegisterOffsetBase.PC0_OFFSET, RegisterOffsetBase.PC1_OFFSET, RegisterOffsetBase.PC2_OFFSET, RegisterOffsetBase.PC3_OFFSET, RegisterOffsetBase.NOT_USED,   RegisterOffsetBase.NOT_USED,   RegisterOffsetBase.NOT_USED,   RegisterOffsetBase.NOT_USED  } ,
            { RegisterOffsetBase.NOT_USED,   RegisterOffsetBase.NOT_USED,   RegisterOffsetBase.NOT_USED,   RegisterOffsetBase.NOT_USED,   RegisterOffsetBase.PC4_OFFSET, RegisterOffsetBase.PC5_OFFSET, RegisterOffsetBase.PC6_OFFSET, RegisterOffsetBase.PC7_OFFSET} ,
        };        

        // Flag for Auto Mode
        private static bool isAutomodeEnabled = false;
        
        // Array to store all 24 pin's enabled alternate function
        private static eAltFunction[,] enabledAlternateFunction = new eAltFunction[4, 8];

        // Variable holds Interrupt Status Register read value
        private static UInt32 interruptStatusRegData = 0;

        // IRQ thread handler
        private static Thread irqThreadHandler = null;

        // Motor Control Output Configuration
        private static ePortName mcsPortName = ePortName.ePortCH;  // Motor Control Output Port Name
        private static ePinNum mcsPinNumber = ePinNum.ePin4;      // Motor Control Output Pin Number

        // Quadrature Input Configuration        
        private static ePortName qdcPortName = ePortName.ePortCL;      // Quadrature Counter Port Name
        private static ePinNum qdcPinNumber = ePinNum.ePin0;          // Quadrature Counter Pin Number
        private static ePortName qdcResetPortName = ePortName.ePortCL; // Quadrature Counter Reset Port Name
        private static ePinNum qdcResetPinNumber = ePinNum.ePin2;     // Quadrature Counter Reset pin Number
        
        // This form control
        private static Main_Form mainForm;

        // String to display start message on log window (IO mapped or Memory mapped)
        private static string startupMessage;

        /*************************************************************************************************************
         * 
         * Public Functions
         * 
         *************************************************************************************************************/         
        /// <Main_Form>
        /// Initialize the MainForm components
        /// </summary>
        public Main_Form()
        {
            InitializeComponent();
            logUI = new AIOWDMLog();
            logUI.Show();
        }

        /// <summary>
        /// Function configures the driver in IO Mapped or Memory mapped mode
        /// </summary>
        /// <param name="cardId"> Card Id from the user selected index</param>
        public UInt16 UpdateStartupConfig(int cardId)
        {
            UInt16 retValue;
            cardNumber = cardId;
            tbLog.Clear();

            startupMessage = "Driver is configured as Memory mapped.\r\n";

            // Configure the driver for IO map or memory map
            retValue = AIOWDM.RelChangeMapping(cardNumber, false);
            if (retValue != 0)
                tbLog.AppendText("Configuration of driver mapping failed, return value: " + retValue.ToString("X"));

            return retValue;
        }        

        /// <summary>
        /// Function to update the Configuration parameter to registers
        /// </summary>
        /// <param name="configParam">Configuration parameters</param>
        public static eStatus UpdatePinConfig(UpdateConfigParam configParam)
        {
            string displayMessage;
            bool isValidRequest = true;
            UInt32 registerAddress = 0;
            UInt32 writeData = 0;
            eStatus returnStatus = eStatus.eSuccess;            

            switch (configParam.AltFun)
            {
                //------------------------------------------------------ INPUT ------------------------------------------------------
                case eAltFunction.eGenIn:
                    // Following Parameters are used for configuration update
                    // configParam.PortName
                    // configParam.PinNumber
                    // configParam.EnableDebounce                     

                    // NOTE: Do not change the direction of a port from here. This routine will be called for all 8 pins of the port
                    // Enable/disable debounce

                    // Check if pin is configured as Quadrature Counter's Index pin
                    if (!((qdcResetPortName == configParam.PortName) && (qdcResetPinNumber == configParam.PinNumber)))                    
                    {
                        // Pin is not configured for reset source for QDC counter.
                        // Enable /Disable debounce based on user selection
                        registerAddress = mainForm.pinConfigRegBaseAddress[(UInt16)configParam.PortName, (UInt16)configParam.PinNumber];
                        writeData = logUI.RelInPortL(cardNumber, registerAddress);

                        if (configParam.EnableDebounce)
                            writeData |= RegData.IDBE_BIT;
                        else
                            writeData &=  ~(UInt32)RegData.IDBE_BIT;

                        // Write updated debounce configuration data to register
                        logUI.RelOutPortL(cardNumber, registerAddress, writeData);
                    }
                    // Port direction has changed. If QDC or MCS is configured, disable the same.
                    // Check if Quadrature input is enabled previously, if enabled disable the same
                    if ((qdcPortName == configParam.PortName) & (qdcPinNumber == configParam.PinNumber))
                    {                        
                        // Disable QDC external reset source
                        if (qdcResetPortName != ePortName.ePortUnknown)
                        {
                            // Clear enable interrupt bit on interrupt enable register
                            mainForm.UpdateInterruptEnableRegister(false, qdcResetPortName, qdcResetPinNumber);
                            // Uncheck the reset on event combobox
                            UncheckResetOnEvent();
                            // Set the QDC reset port and pin number to unknown state
                            qdcResetPortName = ePortName.ePortUnknown;
                            qdcResetPinNumber = ePinNum.ePinUnknown;                            
                        }

                        // Set the QDC port and Pin Name to Unknown
                        qdcPortName = ePortName.ePortUnknown;
                        qdcPinNumber = ePinNum.ePinUnknown;
                        // Enable the reset pin Display
                        DisableQdcResetPin(false, qdcResetPortName, qdcResetPinNumber);

                        // Quadrature counter is disabled now. Clear the IsQdcUsed flag
                        IsQdcUsed = false;

                        // Clear the QDC configuration register
                        logUI.RelOutPortL(cardNumber, RegisterOffsetBase.QDC_CONFIG_REG, 0);
                    }                    
                    // Check if MCS output is configured.
                    if ((mcsPortName == configParam.PortName) & (mcsPinNumber == configParam.PinNumber))
                    {
                        // Disable the MCS configuration on this pin
                        mcsPortName = ePortName.ePortUnknown;
                        mcsPinNumber = ePinNum.ePinUnknown;

                        // Motor control output is disabled now. Clear the IsMcsUsed flag
                        IsMcsUsed = false;

                        // Clear the MCS configuration register
                        logUI.RelOutPortL(cardNumber, RegisterOffsetBase.MCS_CONFIG_REG, 0);
                    }
                    break;

                case eAltFunction.ePulseDetect:
                    // Update the pulse Detect Configuration
                    mainForm.ConfigPulseDetect(configParam);                    
                    break;

                case eAltFunction.eLtoHDetect:
                    // Initialize enable IRQ flag 
                    bool enableIrq = false;
                    // Display dialog to get enable or disable IRQ info from user
                    DialogResult dialogResult = MessageBox.Show("Do you want to enable IRQ for this pin", "Setting", MessageBoxButtons.YesNo);
                    if (dialogResult == DialogResult.Yes)
                        enableIrq = true;  // Set the enable IRQ flag

                    // Get the base address of the pin
                    registerAddress = mainForm.pinConfigRegBaseAddress[(UInt16)configParam.PortName, (UInt16)configParam.PinNumber];

                    // Get the current content of the pin configuration register
                    writeData = logUI.RelInPortL(cardNumber, registerAddress); 

                    // Clear the last Byte 
                    writeData &= ~(UInt32)0x0F;

                    // Set Low to High Event Flag
                    writeData = writeData | RegData.ALT_FUN_L2H;                    

                    if (enableIrq)   // Set the IRQn_EN bit if IRQ is enabled by the user                     
                        writeData |= RegData.IRQ_EN;

                    // Write configuration data to Register
                    logUI.RelOutPortL(cardNumber, registerAddress, writeData);

                    // Enable or disable the IRQ based on User Selection
                    mainForm.UpdateInterruptEnableRegister(enableIrq, configParam.PortName, configParam.PinNumber);                    
                    break;

                case eAltFunction.eCntPulse:
                    // Update the pulse Count Configuration
                    mainForm.ConfigPulseCount(configParam);
                    break;

                case eAltFunction.eMeasPulseHigh:
                    // Update the pulse High Measurement Configuration
                    mainForm.ConfigMeasurePulseDuration(configParam);
                    break;

                case eAltFunction.eMeasFreq:
                    // Update the Frequency Configuration                           
                    mainForm.ConfigMeasureFrequency(configParam);
                    break;

                case eAltFunction.eMeasDutyCycle:
                    // Update the Duty Cycle Configuration
                    mainForm.ConfigMeasureDutyCycle(configParam);                    
                    break;

                case eAltFunction.eCntQuadPulse:
                    // Following Parameters are used for configuration update
                    // configParam.pName
                    // configParam.PinNumber
                    // configParam.AltFun
                    // configParam.ResetCounter
                    // configParam.QdcConfig.IsResetConfig
                    // configParam.QdcConfig.QdcResetPinName
                    // configParam.QdcConfig.QdcResetPinNumber                    

                    // Check if QDC reset configuration is passed                   
                    if (configParam.QdcConfig.IsResetConfig == true)
                    {
                        if (configParam.QdcConfig.QdcResetTriggerType != eConfigParam.eNone)
                        {
                            // Enable Index Pin
                            mainForm.EnableQdcIndexPin(true);

                            // Save Reset port name and pin number locally
                            qdcResetPortName = configParam.QdcConfig.QdcResetPinName;
                            qdcResetPinNumber = configParam.QdcConfig.QdcResetPinNumber;

                            // Disable the Reset pin on UI so that it cannot be changed to other functions
                            DisableQdcResetPin(true, configParam.QdcConfig.QdcResetPinName, configParam.QdcConfig.QdcResetPinNumber);                            
                        }
                        else
                        {
                            // Disable the reset Pin and make it as general input
                            registerAddress = mainForm.pinConfigRegBaseAddress[(UInt16)configParam.PortName, (UInt16)configParam.PinNumber];
                            writeData = 0;
                            // Clear alternate function set on pin
                            logUI.RelOutPortL(cardNumber, registerAddress, writeData);

                            // Clear Enabled Interrupt on pin configParam.PortName[configParam.PinNumber]
                            mainForm.UpdateInterruptEnableRegister(false, qdcResetPortName, qdcResetPinNumber);

                            // Disable the Qdc Index Pin
                            mainForm.EnableQdcIndexPin(false);

                            // Enable the Display
                            DisableQdcResetPin(false, qdcResetPortName, qdcResetPinNumber);

                            // Clear the QDC reset port name and pin number
                            qdcResetPortName = ePortName.ePortUnknown;
                            qdcResetPinNumber = ePinNum.ePinUnknown;

                            // Make this pin as general input
                            configParam.AltFun = eAltFunction.eGenIn;
                        }
                    }
                    else if (configParam.ResetCounter == true)
                    {
                        // Requested for the QDC counter reset
                        if ((qdcPortName == configParam.PortName) && (qdcPinNumber == (configParam.PinNumber - 1)))
                        {
                            // Get current configuration parameter
                            registerAddress = RegisterOffsetBase.QDC_CONFIG_REG;
                            writeData = logUI.RelInPortL(cardNumber, registerAddress);

                            // Set QDC Reset Bit
                            writeData = writeData | RegData.QDC_RST;
                            logUI.RelOutPortL(cardNumber, registerAddress, writeData);

                            // Clear QDC Reset Bit
                            writeData = writeData & ~(UInt32)RegData.QDC_RST;
                            logUI.RelOutPortL(cardNumber, registerAddress, writeData);

                            // Make reset pin as general input pin
                            configParam.AltFun = eAltFunction.eGenIn;
                        }
                    }
                    else
                    {
                        // Check if QDC is available, if available configure the QDC              
                        if ((!IsQdcUsed) || ((qdcPortName == configParam.PortName) && (qdcPinNumber == configParam.PinNumber)))
                        {
                            // Clear all the alternate functions set for this QDC pins
                            registerAddress = mainForm.pinConfigRegBaseAddress[(UInt16)configParam.PortName, (UInt16)configParam.PinNumber];
                            logUI.RelOutPortL(cardNumber, registerAddress, 0);
                            registerAddress = mainForm.pinConfigRegBaseAddress[(UInt16)configParam.PortName, (UInt16)(configParam.PinNumber + 1)];
                            logUI.RelOutPortL(cardNumber, registerAddress, 0);

                            // Assign Port name and Pin Number to the QDC configuration variables
                            IsQdcUsed = true;
                            qdcPortName = configParam.PortName;
                            qdcPinNumber = configParam.PinNumber;

                            //Update Configuration param
                            registerAddress = RegisterOffsetBase.QDC_CONFIG_REG;
                            writeData = (UInt32)RegData.MCS_CTRL | GetQuadraturePairNumber(configParam.PortName, configParam.PinNumber);
                            logUI.RelOutPortL(cardNumber, registerAddress, writeData);
                        }
                    }
                    break;

                //------------------------------------------------------ OUTPUT ------------------------------------------------------
                case eAltFunction.eGenOut:
                    // NOTE: Do not change the direction of a port from here. This routine will be called for all 8 pins of the port

                    // Clear all the alternate function
                    registerAddress = mainForm.pinConfigRegBaseAddress[(UInt16)configParam.PortName, (UInt16)configParam.PinNumber];
                    logUI.RelOutPortL(cardNumber, registerAddress, 0);       // Load the value to Register
             
                    // Check if Motor Control Output is enabled previously, if enabled disable it now
                    if ((mcsPortName == configParam.PortName) & (mcsPinNumber == configParam.PinNumber))
                    {
                        // Clear the MCS configuration register
                        logUI.RelOutPortL(cardNumber, RegisterOffsetBase.MCS_CONFIG_REG, 0);
                        // Clear MCS port name and pin numbers
                        mcsPortName = ePortName.ePortUnknown;
                        mcsPinNumber = ePinNum.ePinUnknown;
                        // MCS is not used, clear the flag
                        IsMcsUsed = false;                        
                    }
                          
                    // Check if Quadrature input is enabled previously, if enabled disable it
                    if ((qdcPortName == configParam.PortName) & (qdcPinNumber == configParam.PinNumber))
                    {                                                
                        if (qdcResetPortName != ePortName.ePortUnknown)
                        {
                            // Uncheck the reset on event combobox
                            UncheckResetOnEvent();

                            // Clear the interrupt enable register bit
                            mainForm.UpdateInterruptEnableRegister(false, qdcResetPortName, qdcResetPinNumber);

                            // Enable the reset Display
                            DisableQdcResetPin(false, qdcResetPortName, qdcResetPinNumber);

                            // Set the QDC reset pin name and number to unknown state
                            qdcResetPortName = ePortName.ePortUnknown;
                            qdcResetPinNumber = ePinNum.ePinUnknown;
                        }

                        // Clear QDC configuration register
                        logUI.RelOutPortL(cardNumber, RegisterOffsetBase.QDC_CONFIG_REG, 0);

                        // Clear QDC port and pin name settings
                        qdcPortName = ePortName.ePortUnknown;
                        qdcPinNumber = ePinNum.ePinUnknown;

                        // Clear the is QDC used flag
                        IsQdcUsed = false;                        
                    }
                    break;

                case eAltFunction.eGenPulseTrain:
                    // Following Parameters are used for configuration update
                    // configParam.PortName
                    // configParam.PinNumber
                    // configParam.PulseTrainConfig.PulseDir
                    // configParam.PulseTrainConfig.PulseLengthMilliSec
                    // configParam.PulseTrainConfig.PulseCount
                    // configParam.PulseTrainConfig.TimeBetweenPulsesMilliSec   

                    // Convert the parameters to register format
                    UInt32 pulseHighDuration = TimemSecToRegData(configParam.PulseTrainConfig.PulseLengthMilliSec); 
                    UInt32 pulseLowDuration = TimemSecToRegData(configParam.PulseTrainConfig.TimeBetweenPulsesMilliSec);
                    UInt32 pulseCount = configParam.PulseTrainConfig.PulseCount;

                    // Validate the configuration parameters
                    if ((pulseHighDuration > UInt16.MaxValue) || (pulseLowDuration > UInt16.MaxValue) || (pulseCount > UInt16.MaxValue))
                    {
                        // Value is out of range                                                        
                        MessageBox.Show(string.Format("{0}{1}{2}"
                                                  , (pulseHighDuration > UInt16.MaxValue) ? "Pulse High Duration is out of Range\r\n" : ""
                                                  , (pulseLowDuration > UInt16.MaxValue) ? "Pulse Low Duration is out of Range\r\n" : ""
                                                  , (pulseCount > UInt16.MaxValue) ? "Pulse Count is out of Range\r\n" : ""),
                                        "Error");
                        UpdateDisplay(configParam, "Out of range");
                        isValidRequest = false;                        
                    }
                    else
                    {
                        // Update the configured parameter on UI
                        displayMessage = (configParam.PulseTrainConfig.PulseLengthMilliSec * 1000) + "μs " + configParam.PulseTrainConfig.PulseCount + " P";
                        UpdateDisplay(configParam, displayMessage);

                        // Set Alternate Function as OPTG
                        if (configParam.PulseTrainConfig.PulseDir == eConfigParam.ePulseDirLow)
                        {
                            registerAddress = mainForm.pinConfigRegBaseAddress[(UInt16)configParam.PortName, (UInt16)configParam.PinNumber];
                            writeData = RegData.ALT_FUN_OPTG & ~(UInt32)RegData.OPTG_DIR;
                            logUI.RelOutPortL(cardNumber, registerAddress, writeData);                      
                        }
                        else if (configParam.PulseTrainConfig.PulseDir == eConfigParam.ePulseDirHigh)
                        {
                            registerAddress = mainForm.pinConfigRegBaseAddress[(UInt16)configParam.PortName, (UInt16)configParam.PinNumber];
                            writeData = RegData.ALT_FUN_OPTG | RegData.OPTG_DIR;
                            logUI.RelOutPortL(cardNumber, registerAddress, writeData);                    
                        }

                        // Update Pulse Length duration
                        registerAddress = mainForm.pinConfigRegBaseAddress[(UInt16)configParam.PortName, (UInt16)configParam.PinNumber] + BitRegisterOffset.PULSE_HI_REG_OFFSET;
                        logUI.RelOutPortL(cardNumber, registerAddress, pulseHighDuration, "write pulse High duration");

                        registerAddress = mainForm.pinConfigRegBaseAddress[(UInt16)configParam.PortName, (UInt16)configParam.PinNumber] + BitRegisterOffset.PULSE_LO_REG_OFFSET;
                        logUI.RelOutPortL(cardNumber, registerAddress, pulseLowDuration, "write pulse Low duration");

                        // Update Pulse Count
                        registerAddress = mainForm.pinConfigRegBaseAddress[(UInt16)configParam.PortName, (UInt16)configParam.PinNumber] + BitRegisterOffset.COUNTER_8BIT_EVENTPULSEOUT;
                        logUI.RelOutPortL(cardNumber, registerAddress, pulseCount, "write pulse count");
                    }
                    break;

                case eAltFunction.eGenDutyCycle:
                    // Following Parameters are used for configuration update
                    // configParam.PortName
                    // configParam.PinNumber
                    // configParam.DutyCycleConfig.TimePeriodMilliSec
                    // configParam.DutyCycleConfig.DutyCycle

                    // Calculate on time & off time from duty cycle & total time period
                    float onTimeMilliSec = (float)(configParam.DutyCycleConfig.DutyCycle * configParam.DutyCycleConfig.TimePeriodMilliSec) / 100;
                    float offTimeMilliSec = configParam.DutyCycleConfig.TimePeriodMilliSec - onTimeMilliSec;

                    UInt32 onTimeRegValue = TimemSecToRegData(onTimeMilliSec);
                    UInt32 offTimeRegValue = TimemSecToRegData(offTimeMilliSec);

                    // Check if the value is out of range
                    if ((onTimeRegValue > UInt16.MaxValue) || (offTimeRegValue > UInt16.MaxValue))
                    {
                        // Value is out of range display error mesage
                        MessageBox.Show("Entered Time Period is out of range", "Error");
                        UpdateDisplay(configParam, "Out of range");
                        isValidRequest = false;                        
                    }
                    else
                    {
                        // Update the configured parameter on UI
                        displayMessage = "DC:" + configParam.DutyCycleConfig.DutyCycle.ToString() + "%";
                        UpdateDisplay(configParam, displayMessage);

                        // Set Alternate Function as OPWM
                        ChangeOutputAltFunction(configParam.PortName, configParam.PinNumber, RegData.ALT_FUN_OPWM);

                        // Load On and Off Period
                        registerAddress = mainForm.pinConfigRegBaseAddress[(UInt16)configParam.PortName, (UInt16)configParam.PinNumber] + BitRegisterOffset.PULSE_LO_REG_OFFSET;
                        logUI.RelOutPortL(cardNumber, registerAddress, offTimeRegValue, "write LO pulse duration for PWM output");
                        registerAddress = mainForm.pinConfigRegBaseAddress[(UInt16)configParam.PortName, (UInt16)configParam.PinNumber] + BitRegisterOffset.PULSE_HI_REG_OFFSET;
                        logUI.RelOutPortL(cardNumber, registerAddress, onTimeRegValue, "write HI pulse duration for PWM output");
                    }
                    break;

                case eAltFunction.eGenQuad:
                    // Update the UI with configured parameters
                    if (configParam.McsConfig.Direction == eConfigParam.eDirForward)
                        displayMessage = "F : " + configParam.McsConfig.OffsetCount + " Steps";
                    else
                        displayMessage = "R : " + configParam.McsConfig.OffsetCount + " Steps";

                    // Update the text box
                    UpdateDisplay(configParam, displayMessage);

                    // Following Parameters are used for configuration update
                    // configParam.PortName
                    // configParam.PinNumber
                    // configParam.McsConfig.Direction
                    // configParam.McsConfig.BaseFreqKhz
                    // configParam.McsConfig.OffsetCount        

                    // Check if MCS is available            
                    if ((!IsMcsUsed) || ((mcsPortName == configParam.PortName) && mcsPinNumber == configParam.PinNumber))
                    {
                        // Set the IsMcsUsed Flag                        
                        IsMcsUsed = true;
                        // Copy port number and pin name to mcsPortName & mcsPinNumber
                        mcsPortName = configParam.PortName;
                        mcsPinNumber = configParam.PinNumber;

                        // Update the frequency data to register
                        registerAddress = RegisterOffsetBase.MCS_FREQ_REG;
                        float timeMilliSec = 1 / configParam.McsConfig.BaseFreqKhz;    // Convert Khz to mSec
                        writeData = (TimemSecToRegData(timeMilliSec) / 2);
                        logUI.RelOutPortL(cardNumber, registerAddress, writeData);

                        // Update the Offset Count data to register
                        registerAddress = RegisterOffsetBase.MCS_COUNT_REG;
                        writeData = configParam.McsConfig.OffsetCount;
                        logUI.RelOutPortL(cardNumber, registerAddress, writeData);

                        // Read current MCS configuration
                        registerAddress = RegisterOffsetBase.MCS_CONFIG_REG;
                        UInt32 configValue = logUI.RelInPortB(cardNumber, registerAddress);

                        // Update Configuration 
                        configValue &= ~(UInt32)0xF;   // Clear Pair bits
                        writeData = configValue | RegData.MCS_SEL | GetQuadraturePairNumber(configParam.PortName, configParam.PinNumber);
                        writeData &= ~(UInt32)RegData.MCS_CTRL;    // Clear the control bit
                        if (configParam.McsConfig.Direction == eConfigParam.eDirForward)
                            writeData |= RegData.MCS_DIR;   // Set the direction bit (Forward)
                        else
                            writeData &= ~(UInt32)RegData.MCS_DIR;  // Clear the direction bit (Reverse)

                        // Write the data to the MCS configuration register
                        logUI.RelOutPortL(cardNumber, registerAddress, writeData);
                    }
                    break;

                default:
                    isValidRequest = false;
                    break;
            }
            if (isValidRequest)
            {
                // Valid request, so make the return status as success
                returnStatus = eStatus.eSuccess;

                // Save the configuration information for local reference
                ConfigurationParam[(int)configParam.PortName, (int)configParam.PinNumber] = configParam;

                // Save the alternate pin function
                enabledAlternateFunction[(int)configParam.PortName, (int)configParam.PinNumber] = configParam.AltFun;

                // QDC counter is pair for signal so, set the next pin as eCntQuadPulse
                if ((configParam.AltFun == eAltFunction.eCntQuadPulse) && ((int)configParam.PinNumber < 7))  
                    enabledAlternateFunction[(int)configParam.PortName, (int)(configParam.PinNumber + 1)] = eAltFunction.eCntQuadPulse;
            }
            else
            {
                // Invalid configuration request return Failure status
                returnStatus = eStatus.eFailure;
            }

            // Return the status
            return returnStatus;
        }

        /// <summary>
        /// Function updates the general output state
        /// </summary>
        /// <param name="portName">Port name</param>
        /// <param name="pinName">Pin Number</param>
        /// <param name="value">Value to be updated</param>
        /// <returns>Status</returns>
        public static eStatus UpdateOutputState(ePortName portName, ePinNum pinNum, bool value)
        {
            eStatus status = eStatus.eSuccess;
            UInt16 regData = 0;

            // Update the configuration in global structure for future reference Save the configuration information for local reference
            ConfigurationParam[(int)portName, (int)pinNum].PortName = portName;
            ConfigurationParam[(int)portName, (int)pinNum].PinNumber = pinNum;
            ConfigurationParam[(int)portName, (int)pinNum].AltFun = eAltFunction.eGenOut;
            ConfigurationParam[(int)portName, (int)pinNum].OutputState = value;
           
            if (portName == ePortName.ePortA)       // Update the Port-A's register
            {
                regData = logUI.RelInPortB(cardNumber, RegisterOffsetBase.PORTA_REG);          // Read the current values of the output port
                regData = mainForm.SetClearPin(regData, (UInt16)pinNum, value);                 // Update the bit which needs to changed
                logUI.RelOutPortB(cardNumber, RegisterOffsetBase.PORTA_REG, (Byte)regData);    // write back to the port register
            }
            else if (portName == ePortName.ePortB)  // Update the Port-B's register
            {
                regData = logUI.RelInPortB(cardNumber, RegisterOffsetBase.PORTB_REG);          // Read the current values of the output port
                regData = mainForm.SetClearPin(regData, (UInt16)pinNum, value);                 // Update the bit which needs to changed
                logUI.RelOutPortB(cardNumber, RegisterOffsetBase.PORTB_REG, (Byte)regData);    // write back to the port register                
            }
            else if ((portName == ePortName.ePortCL) || (portName == ePortName.ePortCH))        // Update the Port-C's register
            {
                regData = logUI.RelInPortB(cardNumber, RegisterOffsetBase.PORTC_REG);          // Read the current values of the output port
                regData = mainForm.SetClearPin(regData, (UInt16)pinNum, value);                 // Update the bit which needs to changed
                logUI.RelOutPortB(cardNumber, RegisterOffsetBase.PORTC_REG, (Byte)regData);    // write back to the port register                
            }
            return status;
        }

        /// <summary>
        /// Function updates the General input status on the UI
        /// </summary>
        /// <param name="portName">Port Name</param>
        /// <param name="pinNum">Port Number</param>
        /// <param name="param">Input Parameter to be updated</param>
        /// <param name="data">Input data to be updated</param>
        public void UpdateInputState(ePortName portName, ePinNum pinNum, eConfigParam param, UInt32 data)
        {
            FlowLayoutPanel portFlowLayoutPanel;

            // Get the handle to the Flow Layout panel
            portFlowLayoutPanel = mainForm.GetFlowLytPanDetails(portName);

            // Update the value of each input pins
            UpdateInputParams(portFlowLayoutPanel, pinNum, param, data);
        }

        /// <summary>
        /// Function Updates the Quadrature Counter (Input) value on the UI
        /// </summary>
        /// <param name="pName">Enumeration to the Port name</param>
        /// <param name="pNum">Enumeration to the pin number</param>
        public static void updateQuadratureInputUI(ePortName pName, ePinNum pNum)
        {

            FlowLayoutPanel flowLytPan;
            ucPinConfig userControl = null;

            // Get the handle to the Flow Layout panel
            flowLytPan = mainForm.GetFlowLytPanDetails(pName);

            try
            {
                MethodInvoker mi = delegate
                {
                    foreach (Control ctl in flowLytPan.Controls)
                    {
                        if (ctl is ucPinConfig)
                        {
                            userControl = (ctl as ucPinConfig);
                            // Quadrature input in 2 wire input signal, update Line-1 UI
                            if (pNum == (ePinNum)userControl.PinNumber)
                            {
                                ucPinConfig.UpdateQuadratureUI(userControl, 1, eAltFunction.eCntQuadPulse);
                            }
                            // Quadrature input in 2 wire input signal, update Line-2 UI
                            if ((pNum + 1) == (ePinNum)userControl.PinNumber)
                            {
                                ucPinConfig.UpdateQuadratureUI(userControl, 2, eAltFunction.eCntQuadPulse);
                            }
                        }
                    }
                };
                mainForm.Invoke(mi);
            }
            catch (Exception exp)
            {
                MessageBox.Show(exp.ToString());
            }
        }

        /// <summary>
        /// Function Updates the Quadrature Counter (Output) value on the UI
        /// </summary>
        /// <param name="pName">Enumeration to the port name</param>
        /// <param name="pNum">Enumeration to the pin number</param>
        public static void updateQuadratureOutputUI(ePortName pName, ePinNum pNum)
        {
            FlowLayoutPanel flowLytPan;
            ucPinConfig userControl = null;

            // Get the handle to the Flow Layout panel
            flowLytPan = mainForm.GetFlowLytPanDetails(pName);

            try
            {
                MethodInvoker mi = delegate
                {
                    foreach (Control ctl in flowLytPan.Controls)
                    {
                        if (ctl is ucPinConfig)
                        {
                            userControl = (ctl as ucPinConfig);
                            // Quadrature input in 2 wire input signal, update Line-1 UI
                            if (pNum == (ePinNum)userControl.PinNumber)
                            {
                                ucPinConfig.UpdateQuadratureUI(userControl, 1, eAltFunction.eGenQuad);
                            }
                            // Quadrature input in 2 wire input signal, update Line-2 UI
                            if ((pNum + 1) == (ePinNum)userControl.PinNumber)
                            {
                                ucPinConfig.UpdateQuadratureUI(userControl, 2, eAltFunction.eGenQuad);
                            }
                        }
                    }
                };
                mainForm.Invoke(mi);
            }
            catch (Exception exp)
            {
                MessageBox.Show(exp.ToString());
            }
        }
        
        /// <summary>
        /// Function Unchecks the Reset on event check box. 
        /// </summary>
        public static void UncheckResetOnEvent ()
        {
            FlowLayoutPanel flowLytPan;
            ucPinConfig userControl = null;

            // Get the control to the flow layout panel
            flowLytPan = mainForm.GetFlowLytPanDetails(qdcPortName);

            try
            {
                MethodInvoker mi = delegate
                {
                    foreach (Control ctl in flowLytPan.Controls)
                    {
                        if (ctl is ucPinConfig)
                        {
                            userControl = (ctl as ucPinConfig);                            
                            if (qdcPinNumber == (ePinNum)userControl.PinNumber)
                            {
                                userControl.UpdateResetOnEventCheckBox(false);  // Uncheck the reset on event check box on User control
                            }                           
                        }
                    }
                };
                mainForm.Invoke(mi);
            }
            catch (Exception exp)
            {
                MessageBox.Show(exp.ToString());
            }
        }

        /// <summary>
        /// Function Returns the Quadrature Counter's Port name and pin Number
        /// </summary>
        /// <returns>Port and pin number of Quadrature Counter</returns>
        public static Tuple<ePortName, ePinNum> GetQdcPortPinDetails()
        {
            ePortName portName = qdcPortName;
            ePinNum pinName = qdcPinNumber;

            return new Tuple<ePortName, ePinNum>(portName, pinName);
        }

        /*************************************************************************************************************
         * 
         * Private Functions
         * 
         *************************************************************************************************************/
        /// <summary>
        /// Finds the connected card details
        /// </summary>
        private void UpdateCardInfo()
        {
            // Local variables for QueryCardInfo():
            UInt32 DeviceID = 0, Base;
            UInt32 NameSize = 256;
            UInt16[] Name = new UInt16[256];
            String strname;

            // Update the log with information - IO mapped or memory mapped mode selected
            tbLog.Text = startupMessage;

            // Get the total number of cards installed:
            Int32 NumCards = AIOWDM.GetNumCards();

            if (NumCards == 0) // no cards present
            {
                statusStrip.Text = "No Card Found, NumCards = 0";

                tbLog.AppendText("No cards were found!   \r\n");
                tbLog.AppendText("This may mean the card is not installed.  \r\n");
                tbLog.AppendText("Check Device Manager for a card and its status.  \r\n");
                tbLog.AppendText("You may consider rebooting your system.  \r\n"); 
                lbDeviceInfo.Text = "Unknown Device";
                toolStripStatusLabel1.Text = "Device not Connected";
            }
            else
            {
                // Validate and store card data:                
                UInt32 status = AIOWDM.QueryCardInfo(cardNumber, out DeviceID, out Base, out NameSize, out strname);
                switch (DeviceID)
                {

                    case DeviceInfo.DEVICE_ID:
                        lbDeviceInfo.Text = DeviceInfo.DEVICE_NAME;
                        toolStripStatusLabel1.Text = DeviceInfo.DEVICE_NAME + "Connected";
                        break;

                    default:
                        lbDeviceInfo.Text = "Unknown Device";
                        toolStripStatusLabel1.Text = "Unknown Device";
                        break;
                }
            }
        }

        /// <summary>
        /// Function called while loading the main form
        /// </summary>
        /// <param name="sender">The object which has raised the event</param>
        /// <param name="e">Additional information about the event</param>
        private void MainForm_Load(object sender, EventArgs e)
        {
            // Update the software version on UI
            this.Text += " " + DeviceInfo.SOFTWARE_VERSION;            
#if !DEBUG
            btTestButton.Visible = false;
#endif    
            btAOPRun.Visible = false;
                    
            // Assign this to the mainForm variable
            mainForm = this;

            // Add direction elements to direction combo box
            string[] portDirection = new string[] { "Input", "Output" };
            cbDirPortA.Items.AddRange(portDirection);
            cbDirPortB.Items.AddRange(portDirection);
            cbDirPortCL.Items.AddRange(portDirection);
            cbDirPortCH.Items.AddRange(portDirection);

            // By Default all the ports are input port 
            cbDirPortA.SelectedIndex = 0;
            cbDirPortB.SelectedIndex = 0;
            cbDirPortCL.SelectedIndex = 0;
            cbDirPortCH.SelectedIndex = 0;

            // Add user controls to the main UI
            UserControlAdd(ePortName.ePortA);
            UserControlAdd(ePortName.ePortB);
            UserControlAdd(ePortName.ePortCL);
            UserControlAdd(ePortName.ePortCH);

            // Initialize the user control with all input ports
            UserControlInit(fLytPanPortA);
            UserControlInit(fLytPanPortB);
            UserControlInit(fLytPanPortCL);
            UserControlInit(fLytPanPortCH);

            // Find and update mPCIe Card info
            UpdateCardInfo();

            // Global reset set to all to default values
            logUI.RelOutPortL(cardNumber, RegisterOffsetBase.RESETS_REG, RegData.GRST_BIT);
            logUI.RelOutPortL(cardNumber, RegisterOffsetBase.RESETS_REG, 0);
            
            // Start the periodic timer             
            PeriodicTimer.Start();

            // Create & Start the thread to handle IRQ's
            irqThreadHandler = new Thread(new ThreadStart(ProcessIrqThread));
            irqThreadHandler.Priority = ThreadPriority.Highest; // Set the priority to Highest
            irqThreadHandler.Start();                           // Start the thread
        }

        /// <summary>
        /// Function get invoked while closing the Main form
        /// </summary>
        /// <param name="sender">The object which has raised the event</param>
        /// <param name="e">Additional information about the event</param>
        private void MainForm_FormClosing(object sender, FormClosingEventArgs e)
        {
            // Close the startup thread
            StartUpWinForm.CloseStartupConfig();

            // Abort the thread            
            irqThreadHandler.Abort();

            // Dispose of unmanaged resources.
            Dispose(true);
            GC.SuppressFinalize(this);                        
        }

        /// <summary>
        /// Adds the user controls to the flow layout panel
        /// </summary>
        /// <param name="pName">enumeration represents port name</param>
        private void UserControlAdd(ePortName pName)
        {
            // Dynamically add Usercontrol to Port A 
            for (int i = 0; i < 8; i++)
            {
                ucPinConfig uc = new ucPinConfig();
                uc.PinNumber = i;
                if (pName == ePortName.ePortA)
                {
                    uc.PortName = 'A';          // Set the port name as 'A'
                    uc.Name = "A" + i;          // set the user control name as "Ai" i is from 0 to 7
                    fLytPanPortA.Controls.Add(uc);      // Add the user control
                }
                else if (pName == ePortName.ePortB)
                {
                    uc.PortName = 'B';          // Set the port name as 'B'
                    uc.Name = "B" + i;          // set the user control name as "Bi" i is from 0 to 7
                    fLytPanPortB.Controls.Add(uc);      // Add the user control
                }
                else if ((pName == ePortName.ePortCL) && (i < 4))       // Port CL
                {
                    uc.PortName = 'C';          // Set the port name as 'C'
                    uc.Name = "C" + i;          // set the user control name as "Ci" i is from 0 to 7
                    fLytPanPortCL.Controls.Add(uc);     // Add the user control
                }
                else if ((pName == ePortName.ePortCH) && (i > 3))       // Port CH
                {
                    uc.PortName = 'C';          // Set the port name as 'C'
                    uc.Name = "C" + i;          // set the user control name as "Ci" i is from 0 to 7
                    fLytPanPortCH.Controls.Add(uc);      // Add the user control
                }
            }
        }

        /// <summary>
        /// Initialize the user control with default values
        /// </summary>
        /// <param name="flyLytPan">control to the FlowLayoutPanel</param>
        private void UserControlInit(FlowLayoutPanel flwLytPan)
        {
            eDirection selectedIndex = eDirection.eInput;
            ucPinConfig userControl = null;
            try
            {
                // Invoke an anonymous method on the thread of the form.
                MethodInvoker mi = delegate
                {
                    // Update Port - A user controls

                    foreach (Control ctl in flwLytPan.Controls)
                    {
                        if (ctl is ucPinConfig)
                        {
                            userControl = (ctl as ucPinConfig);
                            userControl.UpdatePinAlternateFunction(selectedIndex, userControl.PortName, userControl.PinNumber);
                        }
                    }
                };
                this.Invoke(mi);
            }
            catch (Exception exc)
            {
                MessageBox.Show(exc.ToString());
            }
        }

        /// <summary>
        /// Function Resets the Quadrature Counter
        /// </summary>
        private void ResetQdcCounter()
        {
            // Get current configuration parameter
            UInt32 registerAddress = RegisterOffsetBase.QDC_CONFIG_REG;
            UInt32 writeData = logUI.RelInPortL(cardNumber, registerAddress);

            // Set the Reset bit in the QDC configuration register and write to the register
            writeData = writeData | RegData.QDC_RST;
            logUI.RelOutPortL(cardNumber, registerAddress, writeData);

            // Clear the Reset bit in the QDC configuration register and write to the register
            writeData = writeData & ~(UInt32)RegData.QDC_RST;
            logUI.RelOutPortL(cardNumber, registerAddress, writeData);
        }

        /// <summary>
        /// Returns the system date and time in string format
        /// </summary>
        /// <returns>String containing current date and time in "yyyy:MM:dd HH:mm:ss" format </returns>
        private string GetTimeStamp()
        {            
            return (DateTime.Now.ToString("yyyy:MM:dd HH:mm:ss - "));
        }        

        /// <summary>
        /// Thread handles the IRQ's
        /// </summary>
        private void ProcessIrqThread()
        {            
            UInt32 status = 0;
            string message = null;
            UInt32 intStatus = 0;

            for (;;)
            {                 
                 // NOTE:    WaitForIRQ function deadlocks the calling thread until an IRQ occurs or the request is aborted. 
                 //          It's intended for multi-threaded IRQ handling, where the calling application spawns a separate thread to service IRQs quickly.
                 //          
                 //          It returns nonzero if it returned because an IRQ occurred, or zero if it returned because the wait was aborted or there was an error. 
                 //          If the wait was aborted, GetLastError() will return ERROR_OPERATION_ABORTED, and if someone is already waiting for an IRQ on that card, 
                 //          it will return ERROR_INVALID_FUNCTION. (Other errors will return other error codes.)                 
                if (AIOWDM.WaitForIRQ(cardNumber) != 0)
                {
                    interruptStatusRegData = logUI.RelInPortL(cardNumber, RegisterOffsetBase.INT_STATUS_REG);     // Read Interrupt Status Register
                    UInt32 intEnableRegData = logUI.RelInPortL(cardNumber, RegisterOffsetBase.INT_ENABLE_REG);    // Read Interrupt Enable Register                    

                    // Interrupt status has read will be processed below. Clear the pending IRQ's
                    logUI.RelOutPortB(cardNumber, RegisterOffsetBase.GLOBAL_INT_STATUS_CLEAR_REG, RegData.GLOBAL_INT_STATUS_CLEAR);

                    // Logical AND with Interrupt status and Interrupt enable register
                    intStatus = interruptStatusRegData & intEnableRegData; // Process the interrupt only if interrupt bit is set and interrupt is enabled for the pin

                    // Ignore the lower 8-bit on the interrupt status register
                    intStatus = intStatus >> 8;

                    // Walk through the each bit of the interrupt status register and update the UI is interrupt bit is set
                    for (int idx = 0; idx < 24; idx++)
                    {
                        message = null; // Clear the previous message buffer
                        status = intStatus & (UInt32)(1 << idx);
                        if (status != 0)
                        {
                            // Check which port got the interrupt
                            if (idx < 8)
                            {                                                                
                                message = GetTimeStamp() + "Interrupt on Port A[" + idx.ToString() + "]\r\n";   // Display Interrupt message on Log text box
                            }
                            else if (idx < 16)
                            {
                                message = GetTimeStamp() + "Interrupt on Port B[" + (idx - 8).ToString() + "]\r\n";     // Display Interrupt message on Log text box
                            }
                            else if (idx < 20)
                            {
                                // Check if this interrupt needs to clear Quadrature counter
                                if ((qdcResetPortName == ePortName.ePortCL) && (qdcResetPinNumber == (ePinNum)idx))
                                {
                                    // ResetQdcCounter();    // QDC reset is handled By FPGA 
                                }
                                else
                                {
                                    message = GetTimeStamp() + "Interrupt on Port CL[" + (idx - 16).ToString() + "]\r\n";   // Display Interrupt message on Log text box
                                }
                            }
                            else if (idx < 24)
                            {                                
                                message = GetTimeStamp() + "Interrupt on Port CH[" + (idx - 16).ToString() + "]\r\n";   // Display Interrupt message on Log text box                          
                            }

                            // Update the message on UI
                            if (message != null)
                            {
                                Invoke((MethodInvoker)delegate {
                                    tbLog.AppendText(message);
                                });
                            }                            
                        }
                    }                    
                }
            }
        }

        /// <summary>
        /// Function converts the Time (mSec) value to register value
        /// </summary>
        /// <param name="timeMilliSec">Time mSec which needs to converted to Register value</param>
        /// <returns></returns>
        private static UInt32 TimemSecToRegData(float timeMilliSec)
        {
            UInt32 result = (UInt32)((timeMilliSec / 1000) / TIME_ONE_LSB);
            return result;
        }

        /// <summary>
        /// Function Converts the Register value to Time (mSec)
        /// </summary>
        /// <param name="registerData"> Read register data </param>
        /// <returns></returns>
        private static float RegDataToTimemSec(UInt32 registerData)
        {
            float result = (float)((registerData * TIME_ONE_LSB) / 1000);
            return result;
        }

        /// <summary>
        /// Function Configures the pin for Pulse detect Alternate Function
        /// </summary>
        /// <param name="configParam">Structure holds the Pulse detect Configurations. Following Parameters are used,
        ///                             configParam.PortName
        ///                             configParam.PinNumber
        ///                             configParam.PulseDetectConfig.TriggerType
        ///                             configParam.PulseDetectConfig.EnableIrq            
        /// </param>            
        private void ConfigPulseDetect(UpdateConfigParam configParam)
        {            
            UInt32 registerAddress = mainForm.pinConfigRegBaseAddress[(UInt16)configParam.PortName, (UInt16)configParam.PinNumber];
            UInt32 writeData = logUI.RelInPortL(cardNumber, registerAddress);
            // Clear the last Byte 
            writeData &= ~(UInt32)0x0F;

            // Update the Pin Configuration Register                                                                      
            if (configParam.PulseDetectConfig.TriggerType == eConfigParam.eFallingEdge)
                writeData = writeData | RegData.ALT_FUN_H2L;
            else if (configParam.PulseDetectConfig.TriggerType == eConfigParam.eRisingEdge)
                writeData = writeData | RegData.ALT_FUN_L2H;
            else if (configParam.PulseDetectConfig.TriggerType == eConfigParam.eLowPulse)
                writeData = writeData | RegData.ALT_FUN_LOW_Pulse;
            if (configParam.PulseDetectConfig.TriggerType == eConfigParam.eHighPulse)
                writeData = writeData | RegData.ALT_FUN_HIGH_Pulse;

            if (configParam.PulseDetectConfig.EnableIrq)   // Set the EnableIrq bit if IRQ is enabled by the user                     
                writeData |= RegData.IRQ_EN;

            // Write configuration data to Register
            logUI.RelOutPortL(cardNumber, registerAddress, writeData);

            // Set/Clear IRQn bit in IRQ Enable register
            UpdateInterruptEnableRegister(configParam.PulseDetectConfig.EnableIrq, configParam.PortName, configParam.PinNumber);   
        }

        /// <summary>
        /// Function configures the pin for Pulse Counter
        /// </summary>
        /// <param name="configParam">Configuration parameters, following parameters are used
        ///                             configParam.PortName
        ///                             configParam.PinNumber
        ///                             configParam.EventCounterConfig.TriggerType
        ///                             configParam.EventCounterConfig.CounterThreshold
        /// </param>
        private void ConfigPulseCount(UpdateConfigParam configParam)
        {   
            // Update the Threshold Parameter
            UInt32 registerAddress = mainForm.pinConfigRegBaseAddress[(UInt16)configParam.PortName, (UInt16)configParam.PinNumber] + BitRegisterOffset.COUNTER_8BIT_EVENTPULSEOUT;
            UInt32 writeData = (UInt32)(configParam.EventCounterConfig.CounterThreshold);

            // Write configuration data to Register
            logUI.RelOutPortL(cardNumber, registerAddress, writeData, "Write pulse count");
            
            // Get the Base Address of the pin
            registerAddress = mainForm.pinConfigRegBaseAddress[(UInt16)configParam.PortName, (UInt16)configParam.PinNumber];
            writeData = logUI.RelInPortL(cardNumber, registerAddress);

            // Clear the last Byte 
            writeData &= ~(UInt32)0x0F;

            // Update the Pin Configuration Register                                                                      
            if (configParam.EventCounterConfig.TriggerType == eConfigParam.eFallingEdge)
                writeData = writeData | RegData.ALT_FUN_H2L;
            else if (configParam.EventCounterConfig.TriggerType == eConfigParam.eRisingEdge)
                writeData = writeData | RegData.ALT_FUN_L2H;
            else if (configParam.EventCounterConfig.TriggerType == eConfigParam.eLowPulse)
                writeData = writeData | RegData.ALT_FUN_LOW_Pulse;
            if (configParam.EventCounterConfig.TriggerType == eConfigParam.eHighPulse)
                writeData = writeData | RegData.ALT_FUN_HIGH_Pulse;

            // Write configuration data to Register
            logUI.RelOutPortL(cardNumber, registerAddress, writeData);
        }

        /// <summary>
        /// Function Configures the pin for Pulse High and Low duration measurement
        /// </summary>
        /// <param name="configParam">Configuration parameters, following parameters are used
        ///                             configParam.PinNumber
        ///                             configParam.PortName        
        /// </param>
        private void ConfigMeasurePulseDuration(UpdateConfigParam configParam)
        {
            // Get the Base address and read the existing configurations
            UInt32 registerAddress = mainForm.pinConfigRegBaseAddress[(UInt16)configParam.PortName, (UInt16)configParam.PinNumber];
            UInt32 writeData = logUI.RelInPortL(cardNumber, registerAddress);

            // Set the Alternate function as IPHL and write to the register
            writeData = (writeData & ~(UInt32)0xF0) | RegData.ALT_FUN_IPHL;
            logUI.RelOutPortL(cardNumber, registerAddress, writeData);
        }

        /// <summary>
        /// Function Configures the pin for Frequency measurement
        /// </summary>
        /// <param name="configParam">Configuration parameters, following parameters are used
        ///                             configParam.PinNumber
        ///                             configParam.PortName        
        /// </param>
        private void ConfigMeasureFrequency(UpdateConfigParam configParam)
        {
            // Get the current Pin Configuration register data
            UInt32 registerAddress = mainForm.pinConfigRegBaseAddress[(UInt16)configParam.PortName, (UInt16)configParam.PinNumber];
            UInt32 writeData = logUI.RelInPortL(cardNumber, registerAddress);

            // Set IFQM bit in the pin configuration register
            writeData = (writeData & ~(UInt32)0xF0) | RegData.ALT_FUN_IFQM;  
            logUI.RelOutPortL(cardNumber, registerAddress, writeData);
        }

        /// <summary>
        /// Function Configures the pin for Duty cycle measurement
        /// </summary>
        /// <param name="configParam">Configuration parameters, following parameters are used
        ///                             configParam.PinNumber
        ///                             configParam.PortName        
        /// </param>
        private void ConfigMeasureDutyCycle(UpdateConfigParam configParam)
        {
            // Get the current Pin Configuration register data
            UInt32 registerAddress = mainForm.pinConfigRegBaseAddress[(UInt16)configParam.PortName, (UInt16)configParam.PinNumber];
            UInt32 writeData = logUI.RelInPortL(cardNumber, registerAddress);

            // Set IPWM bit in the Pin configuration Register
            writeData = (writeData & ~(UInt32)0xF0) | RegData.ALT_FUN_IPWM;
            logUI.RelOutPortL(cardNumber, registerAddress, writeData);
        }

        /// <summary>
        /// Function sets the output control bit of the specified port & pin
        /// </summary>
        /// <param name="portName"> Enumeration for Port Name </param>
        /// <param name="PinNum"> Enumeration for Pin Number </param>
        public static void SetOutputCtrlBit(ePortName portName, ePinNum PinNum)
        {
            UInt32 writeData = 0;

            if (isAutomodeEnabled)
            {
                MessageBox.Show("Application is running in auto mode. Writing to the Control Register has no effect");
            }
            else
            {               
                // Set the control bit based on the port and pin number
                if (portName == ePortName.ePortA)
                    writeData = (UInt32)(1 << (UInt16)PinNum);
                else if (portName == ePortName.ePortB)
                    writeData = (UInt32)(1 << ((UInt16)PinNum + 8));
                else if ((portName == ePortName.ePortCL) || (portName == ePortName.ePortCH))
                    writeData = (UInt32)(1 << ((UInt16)PinNum + 16));

                // Control bit starts from 8th bit, hence left shift the data by 8
                writeData = writeData << 8;
                // Write the control bit value to the register
                logUI.RelOutPortL(cardNumber, RegisterOffsetBase.CTRL_REG, writeData);
            }
        }

        /// <summary>
        /// Sets the Control Bit for Motor Control Output
        /// </summary>
        /// <param name="pName">Enumeration to the Port name</param>
        /// <param name="pNum">Enumeration to the Pin number</param>
        public static void SetMCSCtrlBit(ePortName pName, ePinNum pNum)
        {
            UInt32 registerAddress = 0;
            UInt32 writeData = 0;

            if ((pName == mcsPortName) & (pNum == mcsPinNumber))
            {
                // Set Motor Control Output Control Bit
                registerAddress = RegisterOffsetBase.MCS_CONFIG_REG;
                UInt32 configValue = logUI.RelInPortL(cardNumber, registerAddress);

                // Set MCS control bit and write the value to the MCS configuration register
                writeData = configValue | RegData.MCS_CTRL;
                logUI.RelOutPortL(cardNumber, registerAddress, writeData);
            }
        }    

        /// <summary>
        /// Updates the supported alternate function options based on the set direction 
        /// </summary>
        /// <param name="flowLytPan">handle for FlowLayoutPanel</param>
        /// <param name="dir">direction in or out </param>
        private void UpdateAlternateFunctions(FlowLayoutPanel flowLytPan, eDirection dir)
        {
            ucPinConfig pinConfigUserControl = null;

            // Update the combo box based on the direction set
            try
            {
                // Invoke an anonymous method on the thread of the form.
                MethodInvoker mi = delegate
                {
                    foreach (Control ctl in flowLytPan.Controls)
                    {
                        if (ctl is ucPinConfig)
                        {
                            pinConfigUserControl = (ctl as ucPinConfig);
                            pinConfigUserControl.UpdatePinAlternateFunction(dir, pinConfigUserControl.PortName, pinConfigUserControl.PinNumber);
                        }
                    }
                };
                this.Invoke(mi);
            }
            catch (Exception exp)
            {
                MessageBox.Show(exp.ToString());
            }
        }

        /// <summary>
        /// Changes the direction of the specified port to the specified direction
        /// </summary>
        /// <param name="portName"> Port name direction to be changed </param>
        /// <param name="dir">Direction input or output</param>
        private void ChangePortDirection(ePortName portName, eDirection dir)
        {
            UInt32 regValue = 0;

            //Read the current Direction configuration
            regValue = logUI.RelInPortB(cardNumber, RegisterOffsetBase.PORT_DIR_REG);

            if (dir == eDirection.eInput)
            {
                // Input - Set bit to make it as input port           
                if (portName == ePortName.ePortA)
                    regValue |= RegData.PORTA_DIR_IN;
                else if (portName == ePortName.ePortB)
                    regValue |= RegData.PORTB_DIR_IN;
                else if (portName == ePortName.ePortCL)
                    regValue |= RegData.PORTCL_DIR_IN;
                else if (portName == ePortName.ePortCH)
                    regValue |= RegData.PORTCH_DIR_IN;

                // Update the port direction value to register
                logUI.RelOutPortB(cardNumber, RegisterOffsetBase.PORT_DIR_REG, (byte)regValue);
            }
            else
            {
                // Output - Clear bit to make it as Output port           
                if (portName == ePortName.ePortA)
                    regValue &= ~RegData.PORTA_DIR_IN;
                else if (portName == ePortName.ePortB)
                    regValue &= ~RegData.PORTB_DIR_IN;
                else if (portName == ePortName.ePortCL)
                    regValue &= ~RegData.PORTCL_DIR_IN;
                else if (portName == ePortName.ePortCH)
                    regValue &= ~RegData.PORTCH_DIR_IN;

                // Change the direction 
                logUI.RelOutPortB(cardNumber, RegisterOffsetBase.PORT_DIR_REG, (byte)regValue);

                // Change the output pin state to Low (Default State)
                if (portName == ePortName.ePortA)
                {
                    logUI.RelOutPortB(cardNumber, RegisterOffsetBase.PORTA_REG, (byte)0x00);
                }
                else if (portName == ePortName.ePortB)
                {
                    logUI.RelOutPortB(cardNumber, RegisterOffsetBase.PORTB_REG, (byte)0x00);
                }
                else if (portName == ePortName.ePortCL)
                {
                    regValue = logUI.RelInPortB(cardNumber, RegisterOffsetBase.PORTC_REG);
                    logUI.RelOutPortB(cardNumber, RegisterOffsetBase.PORTC_REG, (byte)(regValue & 0xF0));    // Clear the lower nibble of port C register                    
                }
                else if (portName == ePortName.ePortCH)
                {
                    regValue = logUI.RelInPortB(cardNumber, RegisterOffsetBase.PORTC_REG);
                    logUI.RelOutPortB(cardNumber, RegisterOffsetBase.PORTC_REG, (byte)(regValue & 0x0F));    // Clear the Higher nibble of port C register
                }
            }
        }

        /// <summary>
        /// Function returns the handle to the Flow Layout panel
        /// </summary>
        /// <param name="pName">Enumeration for the port number</param>
        /// <returns="FlowLayoutPanel">Returns the control to the flow layout panel</returns>
        private FlowLayoutPanel GetFlowLytPanDetails(ePortName pName)
        {
            FlowLayoutPanel flowLayoutPanelControl = null;

            if (mainForm == null)
                return flowLayoutPanelControl;

            if (pName == ePortName.ePortA)
                flowLayoutPanelControl = mainForm.fLytPanPortA;
            else if (pName == ePortName.ePortB)
                flowLayoutPanelControl = mainForm.fLytPanPortB;
            else if (pName == ePortName.ePortCL)
                flowLayoutPanelControl = mainForm.fLytPanPortCL;
            else
                flowLayoutPanelControl = mainForm.fLytPanPortCH;

            return flowLayoutPanelControl;
        }

        /// <summary>
        /// Function Updates the read input parameter on UI
        /// </summary>
        /// <param name="pName">Port Name</param>
        /// <param name="pNum">Pin Number</param>
        /// <param name="altFun">Input Alternate function to be updated</param>
        /// <param name="msgToDisplay1">Message to be displayed on text box - 1</param>
        /// <param name="msgToDisplay2">Message to be displayed on text box - 2 (Used only for measure pulse high/low)</param>
        private void UIUpdateInput(ePortName pName, ePinNum pNum, eAltFunction altFun, string msgToDisplay1, string msgToDisplay2)
        {
            ucPinConfig usrCtrl;
            FlowLayoutPanel flowLytPan;

            flowLytPan = GetFlowLytPanDetails(pName);
            if (flowLytPan == null)
                return;

            try
            {
                MethodInvoker mi = delegate
                {
                    foreach (Control ctl in flowLytPan.Controls)
                    {
                        if (ctl is ucPinConfig)
                        {
                            usrCtrl = (ctl as ucPinConfig);
                            if ((ePinNum)usrCtrl.PinNumber == pNum)
                            {
                                switch (altFun)
                                {
                                    case eAltFunction.eGenIn:   
                                        // General Input message update High/Low on corresponding text box
                                        usrCtrl.UpdateGenInState(msgToDisplay1);
                                        break;

                                    case eAltFunction.ePulseDetect:
                                    case eAltFunction.eLtoHDetect:
                                        usrCtrl.UpdateDisplayConfig(msgToDisplay1);
                                        break;

                                    case eAltFunction.eCntPulse:
                                        usrCtrl.UpdateFrequency(msgToDisplay1);                                        
                                        break;

                                    case eAltFunction.eMeasPulseHigh:
                                        usrCtrl.UpdatePulseHighMeasure(msgToDisplay1, msgToDisplay2);                                        
                                        break;

                                    case eAltFunction.eMeasFreq:
                                        usrCtrl.UpdateFrequency(msgToDisplay1);
                                        break;
                                    case eAltFunction.eMeasDutyCycle:
                                        usrCtrl.UpdateDutyCycleValue(msgToDisplay1);
                                        break;
                                    default:
                                        break;
                                }
                            }
                            else if ((ePinNum)usrCtrl.PinNumber == (pNum + 1) && (altFun == eAltFunction.eCntQuadPulse))    // For quadrature input Value has to be updated in the pin+1 user control
                            {                                
                                // Quadrature Pulse Count message, update it on QDC line 2 text box
                                usrCtrl.UpdateDisplayConfig(msgToDisplay1);
                            }
                        }
                    }
                };
                mainForm.Invoke(mi);
            }
            catch (Exception exp)
            {
                MessageBox.Show(exp.ToString());
            }
        }

        /// <summary>
        /// Handler for port-A direction change event
        /// Function updates the Alternate functions for input & output ports
        /// </summary>
        /// <param name="sender">The object which has raised the event</param>
        /// <param name="e">Additional information about the event</param>
        private void cbDirPortA_SelectedIndexChanged(object sender, EventArgs e)
        {
            ComboBox comboBoxControl = (ComboBox)sender;            
            UpdateAlternateFunctions(fLytPanPortA, (eDirection)comboBoxControl.SelectedIndex);  // Based on the direction update the User control
            ChangePortDirection(ePortName.ePortA, (eDirection)comboBoxControl.SelectedIndex);   // Change the Port-A's Direction            
        }

        /// <summary>
        /// Handler for port-B direction change event
        /// Function updates the Alternate functions for input & output ports
        /// </summary>
        /// <param name="sender">The object which has raised the event</param>
        /// <param name="e">Additional information about the event</param>
        private void cbDirPortB_SelectedIndexChanged(object sender, EventArgs e)
        {
            ComboBox comboBoxControl = (ComboBox)sender;
            UpdateAlternateFunctions(fLytPanPortB, (eDirection)comboBoxControl.SelectedIndex);  // Based on the direction update the User control
            ChangePortDirection(ePortName.ePortB, (eDirection)comboBoxControl.SelectedIndex);   // Change the Port-B's Direction
        }

        /// <summary>
        /// Handler for port-CL direction change event
        /// Function updates the Alternate functions for input & output ports
        /// </summary>
        /// <param name="sender">The object which has raised the event</param>
        /// <param name="e">Additional information about the event</param>
        private void cbDirPortCL_SelectedIndexChanged(object sender, EventArgs e)
        {
            ComboBox comboBoxControl = (ComboBox)sender;
            UpdateAlternateFunctions(fLytPanPortCL, (eDirection)comboBoxControl.SelectedIndex);  // Based on the direction update the User control
            ChangePortDirection(ePortName.ePortCL, (eDirection)comboBoxControl.SelectedIndex);   // Change the Port-CL's Direction            
        }

        /// <summary>
        /// Handler for port-CH direction change event
        /// Function updates the Alternate functions for input & output ports
        /// </summary>
        /// <param name="sender">The object which has raised the event</param>
        /// <param name="e">Additional information about the event</param>
        private void cbDirPortCH_SelectedIndexChanged(object sender, EventArgs e)
        {
            ComboBox comboBoxControl = (ComboBox)sender;           
            UpdateAlternateFunctions(fLytPanPortCH, (eDirection)comboBoxControl.SelectedIndex);  // Based on the direction update the User control
            ChangePortDirection(ePortName.ePortCH, (eDirection)comboBoxControl.SelectedIndex);   // Change the Port-CH's Direction            
        }

        /// <summary>
        /// Handler for Clear Log button click event - Clears the log text
        /// </summary>
        /// <param name="sender">The object which has raised the event</param>
        /// <param name="e">Additional information about the event</param>
        private void btClearLog_Click(object sender, EventArgs e)
        {
            DialogResult dialogResult = MessageBox.Show("This will clear the log data. Do you want to continue?", "Warning", MessageBoxButtons.YesNo);
            if (dialogResult == DialogResult.Yes)            
                tbLog.Clear();  // Clear the log text            
        }

        /// <summary>
        /// Handler for Auto mode checkbox change event, Function Set/clear's the AOP bit        
        /// </summary>
        /// <param name="sender">The object which has raised the event</param>
        /// <param name="e">Additional information about the event</param>
        private void chkbxEnableAutomode_CheckedChanged(object sender, EventArgs e)
        {
            UInt32 registerAddress;
            UInt32 writeData;

            // Update AOP Enable register
            registerAddress = RegisterOffsetBase.INT_ENABLE_REG;
            writeData = logUI.RelInPortL(cardNumber, registerAddress);

            if (chkbxEnableAutomode.Checked == true)
            {
                isAutomodeEnabled = true;       // Set the Auto mode enabled Flag            
                writeData |= RegData.AOP_BIT;   // Set the AOP Bit
            }
            else
            {
                isAutomodeEnabled = false;      // Clear the Auto mode enabled Flag                
                writeData &= ~RegData.AOP_BIT;  // Clear the AOP Bit
            }
            // Write the updated value to the register
            logUI.RelOutPortL(cardNumber, registerAddress, writeData);
        }

        /// <summary>
        /// Periodic Timer of interval 1 Sec. Reads all the register and update the value on UI
        /// </summary>
        /// <param name="sender">The object which has raised the event</param>
        /// <param name="e">Additional information about the event</param>
        private void PeriodicTimer_Tick(object sender, EventArgs e)
        {
            UInt32[] portBase = { RegisterOffsetBase.PORTA_REG, RegisterOffsetBase.PORTB_REG, RegisterOffsetBase.PORTC_REG, RegisterOffsetBase.PORTC_REG };
            UInt32 regReadValue = 0;
            UInt32 regAddress = 0;
            UInt32 intStatusData = 0;
            UInt32 portStatus = 0;
            UInt16 portIndex = 0;
            UInt16 pinIndex = 0;
            bool isPortRegRead = false;
            bool isIntStatusBitSet = false;
            bool isIntStatusRegRead = false;
            string displayMessage1 = null;
            string displayMessage2 = null;                            

            for (portIndex = 0; portIndex < 4; portIndex++)    // Walk through all 4 ports
            {
                isPortRegRead = false;      //reset the flag as reading the next port
                for (pinIndex = 0; pinIndex < 8; pinIndex++)   // Walk through all 8 pins
                {
                    switch (enabledAlternateFunction[portIndex, pinIndex])
                    {
                        case eAltFunction.eGenIn:
                            displayMessage1 = null;            // Clear the message
                            if (!isPortRegRead)  
                            {
                                // Read the port register value only one time per port
                                portStatus = logUI.RelInPortB(cardNumber, portBase[portIndex], "tick generic in");
                                isPortRegRead = true;
                            }

                            if ((portStatus & (1 << (int)pinIndex)) == 0)   // Check if bit is set and update the message accordingly
                                displayMessage1 = "Low";
                            else
                                displayMessage1 = "High";

                            break;

                        case eAltFunction.ePulseDetect:
                            displayMessage1 = "--";        // Clear the display message
                            isIntStatusBitSet = false;     // Clear the interrupt set flag                           
                            if (!isIntStatusRegRead) 
                            {
                                // Since Interrupt status register is clear on read, read only once and process the bits
                                intStatusData = logUI.RelInPortL(cardNumber, RegisterOffsetBase.INT_STATUS_REG, "tick pulse detect") | interruptStatusRegData;
                                isIntStatusRegRead = true;
                            }
                                
                            // Check for interrupt status                             
                            if ((ePortName)portIndex == ePortName.ePortA)
                            {                                
                                if ((((intStatusData >> 8) & 0xFF) & (1 << pinIndex)) != 0)      // Check if interrupt status bit is set
                                {
                                    isIntStatusBitSet = true;                                    // Set the interrupt status flag
                                    interruptStatusRegData &= ~((UInt32)(1 << (pinIndex + 8)));  // Clear processed bit
                                }                          
                                    
                            }
                            else if ((ePortName)portIndex == ePortName.ePortB)
                            {                                
                                if ((((intStatusData >> 16) & 0xFF) & (1 << pinIndex)) != 0)    // Check if interrupt status bit is set
                                {
                                    isIntStatusBitSet = true;                                    // Set the interrupt status flag
                                    interruptStatusRegData &= ~((UInt32)(1 << (pinIndex + 16))); // Clear processed bit
                                }
                            }
                            else if (((ePortName)portIndex == ePortName.ePortCL) || ((ePortName)portIndex == ePortName.ePortCH))
                            {                                
                                if ((((intStatusData >> 24) & 0xFF) & (1 << pinIndex)) != 0)    // Check if interrupt status bit is set
                                {
                                    isIntStatusBitSet = true;                                    // Set the interrupt status flag
                                    interruptStatusRegData &= ~((UInt32)(1 << (pinIndex + 24))); // Clear processed bit
                                }
                            }

                            if (isIntStatusBitSet)
                            {
                                //Interrupt is set, update the UI based on the pin configuration 
                                // Read pin configuration register
                                regReadValue = logUI.RelInPort(cardNumber, pinConfigRegBaseAddress[portIndex, pinIndex], "check config bit");

                                if ((regReadValue & RegData.ALT_FUN_H2L) != 0)
                                    displayMessage1 = "Falling Edge";
                                else if ((regReadValue & RegData.ALT_FUN_L2H) != 0)
                                    displayMessage1 = "Rising Edge";
                                else if ((regReadValue & RegData.ALT_FUN_LOW_Pulse) != 0)
                                    displayMessage1 = "Low Pulse";
                                else if ((regReadValue & RegData.ALT_FUN_HIGH_Pulse) != 0)
                                    displayMessage1 = "High Pulse";
                                else
                                    displayMessage1 = "--";
                            }
                            break;

                        case eAltFunction.eLtoHDetect:
                            displayMessage1 = "--";    // Clear the message                            
                            if (!isIntStatusRegRead)
                            {
                                // Since Interrupt status register is clear on read, read only once and process the bits
                                intStatusData = logUI.RelInPortL(cardNumber, RegisterOffsetBase.INT_STATUS_REG, "check IRQ status bits") | interruptStatusRegData;
                                isIntStatusRegRead = true;
                            }

                            // Check which pin has generated the IRQ
                            if ((ePortName)portIndex == ePortName.ePortA)
                            {                                
                                if ((((intStatusData >> 8) & 0xFF) & (1 << pinIndex)) != 0)     // Check if Interrupt status bit set
                                {
                                    displayMessage1 = "Rising Edge";
                                    interruptStatusRegData &= ~((UInt32)(1 << (pinIndex + 8))); // Clear processed bit
                                }
                            }
                            else if ((ePortName)portIndex == ePortName.ePortB)
                            {                                
                                if ((((intStatusData >> 16) & 0xFF) & (1 << pinIndex)) != 0)    // Check if Interrupt status bit set
                                {
                                    displayMessage1 = "Rising Edge";
                                    interruptStatusRegData &= ~((UInt32)(1 << (pinIndex + 16))); // Clear processed bit
                                }
                            }
                            else if (((ePortName)portIndex == ePortName.ePortCL) || ((ePortName)portIndex == ePortName.ePortCH))
                            {                                
                                if ((((intStatusData >> 24) & 0xFF) & (1 << pinIndex)) != 0)    // Check if Interrupt status bit set
                                {
                                    displayMessage1 = "Rising Edge";
                                    interruptStatusRegData &= ~((UInt32)(1 << (pinIndex + 24))); // Clear processed bit
                                }
                            }                            
                            break;

                        case eAltFunction.eCntPulse:
                            regAddress = pinConfigRegBaseAddress[portIndex, pinIndex] + BitRegisterOffset.EVENT_REG_OFFSET; // Get the IEVD register address for current pin
                            regReadValue = logUI.RelInPortL(cardNumber, regAddress, "read Event Status Register");  // Read IEVD register
                            if ((regReadValue & RegData.ECOFF_BIT) != 0)            // Check if Overflow bit is set
                            {
                                displayMessage1 = "Overflow";                   // Overflow flag is set. Display as overflow
                            }
                            else
                            {                                
                                regReadValue = regReadValue >> 24;
                                displayMessage1 = regReadValue.ToString();      // Update the read pulse count value on display
                            }                                                            
                                                                                                           
                            break;

                        case eAltFunction.eMeasPulseHigh:
                            regAddress = pinConfigRegBaseAddress[portIndex, pinIndex] + BitRegisterOffset.PULSE_HI_REG_OFFSET;  // Get the IPHL register address for current pin
                            uint PulseHighCounts = logUI.RelInPortL(cardNumber, regAddress, " Read Pulse Width High Duration ");    // Read Pulse Width High & Low Duration                    
                            regAddress = pinConfigRegBaseAddress[portIndex, pinIndex] + BitRegisterOffset.PULSE_LO_REG_OFFSET;  // Get the IPHL register address for current pin
                            uint PulseLowCounts = logUI.RelInPortL(cardNumber, regAddress, " Read Pulse Width Low Duration ");    // Read Pulse Width High & Low Duration                    

                            // Convert the register value to time mSec
                            float pulseHighDurationMilliSec = RegDataToTimemSec(PulseHighCounts);
                            float pulseLowDurationMilliSec = RegDataToTimemSec(PulseLowCounts);

                            // Construct the High Pulse Duration String
                            if ((UInt16)(pulseHighDurationMilliSec * 1000000000) == 0)  // Check if time can be displayed in nano sec
                                displayMessage1 = Math.Round((pulseHighDurationMilliSec * 1000000000000), 2).ToString() + " ns";    // Convert and display message in Nano seconds
                            else
                                displayMessage1 = Math.Round((pulseHighDurationMilliSec * 1000000000), 2).ToString() + " us";       // Convert and display message in micro seconds

                            // Construct the Low Pulse Duration String
                            if ((UInt16)(pulseLowDurationMilliSec * 1000000000) == 0)   // Check if time can be displayed in nano sec
                                displayMessage2 = Math.Round((pulseLowDurationMilliSec * 1000000000000), 2).ToString() + " ns";     // Convert and display message in Nano seconds
                            else
                                displayMessage2 = Math.Round((pulseLowDurationMilliSec * 1000000000), 2).ToString() + " us";        // Convert and display message in micro seconds

                            break;

                        case eAltFunction.eMeasDutyCycle:
                            regAddress = pinConfigRegBaseAddress[portIndex, pinIndex] + BitRegisterOffset.PWM_LO_MEASURED_REG_OFFSET;
                            uint PWMLoReadValue = logUI.RelInPortL(cardNumber, regAddress, "read PWM Lo duration counter");
                            regAddress = pinConfigRegBaseAddress[portIndex, pinIndex] + BitRegisterOffset.PWM_HI_MEASURED_REG_OFFSET;
                            uint PWMHiReadValue = logUI.RelInPortL(cardNumber, regAddress, "read PWM Hi duration counter");

                            // Convert the value to %
                            float pulseOnTimeMilliSec = RegDataToTimemSec(PWMHiReadValue);
                            float pulseOffTimeMilliSec = RegDataToTimemSec(PWMLoReadValue);
                            if (pulseOnTimeMilliSec != 0)
                            {
                                // Calculate Duty cycle value and convert it to string               
                                displayMessage1 = Math.Round((((pulseOnTimeMilliSec) / (pulseOnTimeMilliSec + pulseOffTimeMilliSec)) * 100), 2).ToString();
                            }
                            else
                                displayMessage1 = "--";                                                   
                            break;

                        case eAltFunction.eMeasFreq:
                            regAddress = pinConfigRegBaseAddress[portIndex, pinIndex] + BitRegisterOffset.FREQUENCY_COUNTER_REG_OFFSET;
                            regReadValue = logUI.RelInPortL(cardNumber, regAddress, "read Frequency Counter");

                            double freqHz = 1 / RegDataToTimemSec(regReadValue);
                            if (freqHz == 0)
                            {
                                // Error indicator (value is out of range or 0)
                                displayMessage1 = "--";      // This is to avoid displaying infinity in on the UI
                            }
                            else
                            {
                                freqHz = freqHz / 1000;
                                if (freqHz >= 1000000)
                                    displayMessage1 = (Math.Round((freqHz / 1000000), 2)).ToString() + " MHz";    // Display value in MHz                               
                                else if (freqHz >= 1000)
                                    displayMessage1 = (Math.Round((freqHz / 1000), 2)).ToString() + " KHz";    // Display value in KHz                                   
                                else
                                    displayMessage1 = Math.Round(freqHz, 4).ToString() + " Hz";         // Display value in Hz                                
                            }
                            break;                        

                        case eAltFunction.eCntQuadPulse:
                            Int32 quadratureCount = 0;  // Variable should be signed integer. Quadrature count value can be negative
                            quadratureCount = (Int32)logUI.RelInPortL(cardNumber, RegisterOffsetBase.QDC_COUNTER_REG, "Read QDC");
                            displayMessage1 = quadratureCount.ToString();                        
                            break;

                        default:
                            break;
                    }
                    // Update the message on User Interface
                    UIUpdateInput((ePortName)portIndex, (ePinNum)pinIndex, enabledAlternateFunction[portIndex, pinIndex], displayMessage1, displayMessage2);
                }
            }
        }

        /// <summary>
        /// Clear Counters button click event handler
        /// </summary>
        /// <param name="sender">The object which has raised the event</param>
        /// <param name="e">Additional information about the event</param>
        private void btClearCounter_Click(object sender, EventArgs e)
        {
            UInt32 registerAddress;
            UInt32 writeData;

            DialogResult dialogResult = MessageBox.Show("This will reset all the counter values. Do you want to continue?", "Warning", MessageBoxButtons.YesNo);
            if (dialogResult == DialogResult.Yes)
            {
                // Update Interrupt Enable register value
                registerAddress = RegisterOffsetBase.INT_ENABLE_REG;
                writeData = logUI.RelInPortL(cardNumber, registerAddress, "Read IRQ enable register");

                // Set Global Counter Reset bit
                registerAddress = RegisterOffsetBase.RESETS_REG;
                writeData |= RegData.GRSTC_BIT;
                logUI.RelOutPortL(cardNumber, registerAddress, writeData, "write global counter reset bit");

                // Clear Global Counter Reset bit
                writeData &= ~(UInt32)RegData.GRSTC_BIT;
                logUI.RelOutPortL(cardNumber, registerAddress, writeData, "clear global counter reset bit");
            }
        }        

        /// <summary>
        /// Function used to update the input parameters on the UI
        /// </summary>
        /// <param name="configParam">Configuration Paramter</param>
        /// <param name="msg">Message to be updated</param>
        private static void UpdateDisplay(UpdateConfigParam configParam, string msg)
        {
            ucPinConfig usrCtrl;
            FlowLayoutPanel flowLytPan = null;

            // Get the control to the flow layout panel based on the Port name
            if (configParam.PortName == ePortName.ePortA)
                flowLytPan = mainForm.fLytPanPortA;
            else if (configParam.PortName == ePortName.ePortB)
                flowLytPan = mainForm.fLytPanPortB;
            else if (configParam.PortName == ePortName.ePortCL)
                flowLytPan = mainForm.fLytPanPortCL;
            else
                flowLytPan = mainForm.fLytPanPortCH;

            // Invoke an anonymous method on the thread of the form.
            try
            {
                MethodInvoker mi = delegate
                                    {
                                        foreach (Control ctl in flowLytPan.Controls)
                                        {
                                            if (ctl is ucPinConfig)
                                            {
                                                usrCtrl = (ctl as ucPinConfig);
                                                if (configParam.PinNumber == (ePinNum)usrCtrl.PinNumber)
                                                {
                                                    usrCtrl.UpdateDisplayConfig(msg);   // Update the input parameter on User control
                                                }
                                            }
                                        }
                                    };
                mainForm.Invoke(mi);

            }
            catch (Exception exp)
            {
                MessageBox.Show(exp.ToString());
            }
        }

        /// <summary>
        /// Function modifies the output alternate defined pin alternate functionality
        /// </summary>
        /// <param name="pName">Enumeration to the port name</param>
        /// <param name="pNum">Enumeration to the pin number</param>
        /// <param name="data">Alternate function bit needs to be set</param>
        private static void ChangeOutputAltFunction(ePortName pName, ePinNum pNum, UInt32 writeData)
        {            
            UInt32 registerAddress = mainForm.pinConfigRegBaseAddress[(UInt16)pName, (UInt16)pNum];   // Get the pin's base address      
            logUI.RelOutPortL(cardNumber, registerAddress, writeData, "write updated value to alt function register");     //Write updated value to register
        }

        /// <summary>
        /// Function Returns the pin pair number for the quadrature input/output signals
        /// </summary>
        /// <param name="pName">Enumeration to the port name</param>
        /// <param name="pNum">Enumeration to the Pin number</param>
        /// <returns></returns>
        private static UInt16 GetQuadraturePairNumber(ePortName pName, ePinNum pNum)
        {
            UInt16 pairNumber = 0;

            try
            {
                if (pName == ePortName.ePortA)
                    pairNumber = (UInt16)((UInt16)pNum / 2 + 1);    // Get the pair number for Port A
                else if (pName == ePortName.ePortB)
                    pairNumber = (UInt16)(((UInt16)pNum / 2) + 5);  // Get the pair number for Port B
                else if ((pName == ePortName.ePortCL) || (pName == ePortName.ePortCH))
                    pairNumber = (UInt16)(((UInt16)pNum / 2) + 9);  // Get the pair number for Port CL & Port CH
            }
            catch
            {
                pairNumber = 0xFF;
                MessageBox.Show("Failed to get pair number from pName and pNum");
            }            
            return pairNumber;
        }

        /// <summary>
        /// Function Enable/disable the QDC reset pin
        /// </summary>
        /// <param name="isDisable">if value is true function disables the pin</param>
        /// <param name="pName">Port name</param>
        /// <param name="pNum">Pin number</param>
        private static void DisableQdcResetPin(bool isDisable, ePortName pName, ePinNum pNum)
        {
            ucPinConfig inputPinUserCtrl = null;
            FlowLayoutPanel flowLytPan = null;

            flowLytPan = mainForm.GetFlowLytPanDetails(pName);
            if (flowLytPan == null)
                return;

            // Invoke an anonymous method on the thread of the form.
            try
            {
                MethodInvoker mi = delegate
                {
                    foreach (Control ctl in flowLytPan.Controls)
                    {
                        if (ctl is ucPinConfig)
                        {
                            inputPinUserCtrl = (ctl as ucPinConfig);
                            if (pNum == (ePinNum)inputPinUserCtrl.PinNumber)
                            {
                                inputPinUserCtrl.UpdateQdcResetPin(inputPinUserCtrl, isDisable);    // Enable or disable the QDC Reset Pin
                            }
                        }
                    }
                };
                mainForm.Invoke(mi);
            }
            catch (Exception exp)
            {
                MessageBox.Show(exp.ToString());
            }
        }

        /// <summary>
        /// Function Sets/Clears the specified bit of the data field
        /// </summary>
        /// <param name="data"> data which needs to be manipulated</param>
        /// <param name="pinNum"> pinNum indicates bit position </param>
        /// <param name="setBit"> true is bit needs to be set, else false</param>
        /// <returns></returns>
        private UInt16 SetClearPin(UInt16 data, UInt16 pinNum, bool setBit)
        {
            UInt16 result = 0;

            if (setBit)            
                result = (UInt16)((UInt16)data | (UInt16)(1 << pinNum));            // Set the requested bit
            else            
                result = (UInt16)((UInt16)data & (UInt16)~(1 << pinNum));           // Clear the requested bit

            return result;
        }       

        /// <summary>
        /// Updates the input parameters on the User interface
        /// </summary>
        /// <param name="flowLytPan"> control to the FlowLayoutPanel</param>
        /// <param name="pinNum">Enumeration to the pin Number</param>
        /// <param name="param">Configuration parameter to be updated</param>
        /// <param name="data">data needs to be updated</param>
        private void UpdateInputParams(FlowLayoutPanel flowLytPan, ePinNum pinNum, eConfigParam param, UInt32 data)
        {
            ucPinConfig inputPinUserCtrl = null;

            // Invoke an anonymous method on the thread of the form.
            try
            {
                MethodInvoker mi = delegate
                {
                    foreach (Control ctl in flowLytPan.Controls)
                    {
                        if (ctl is ucPinConfig)
                        {
                            inputPinUserCtrl = (ctl as ucPinConfig);
                            if (pinNum == (ePinNum)inputPinUserCtrl.PinNumber)
                            {
                                inputPinUserCtrl.UpdateInputValue(param, data);
                            }
                        }
                    }
                };
                this.Invoke(mi);
            }
            catch (Exception exp)
            {
                MessageBox.Show(exp.ToString());
            }
        }

        /// <summary>
        /// Function Sets or Clears the bit of Interrupt Enable register
        /// </summary>
        /// <param name="enableInterrupt">true will enables the interrupt and false will disables the interrupt</param>
        /// <param name="pName">Port Name</param>
        /// <param name="pNum">Pin Number</param>
        private void UpdateInterruptEnableRegister(bool enableInterrupt, ePortName pName, ePinNum pNum)
        {
            if (!((pName == ePortName.ePortUnknown) && (pNum == ePinNum.ePinUnknown)))
            {
                // Read Interrupt Enable register value
                UInt32 registerAddress = RegisterOffsetBase.INT_ENABLE_REG;
                UInt32 writeData = logUI.RelInPortL(cardNumber, registerAddress, "read IRQ en register");

                if (enableInterrupt)
                {
                    // Set IRQn bit in IRQ Enable register
                    if (pName == ePortName.ePortA)
                        writeData |= ((UInt32)1 << ((UInt16)pNum + 8));
                    else if (pName == ePortName.ePortB)
                        writeData |= ((UInt32)1 << ((UInt16)pNum + 16));
                    else if ((pName == ePortName.ePortCL) || (pName == ePortName.ePortCH))
                        writeData |= ((UInt32)1 << ((UInt16)pNum + 24));

                    // Enable the Global Interrupt
                    logUI.RelOutPortB(cardNumber, RegisterOffsetBase.GLOBAL_INT_ENABLE_REG, RegData.GLOBAL_INT_ENABLE, "enable global interrupt");
                }
                else
                {
                    // Clear IRQn bit in IRQ Enable register
                    if (pName == ePortName.ePortA)
                        writeData &= ~((UInt32)1 << ((UInt16)pNum + 8));
                    else if (pName == ePortName.ePortB)
                        writeData &= ~((UInt32)1 << ((UInt16)pNum + 16));
                    else if ((pName == ePortName.ePortCL) || (pName == ePortName.ePortCH))
                        writeData &= ~((UInt32)1 << ((UInt16)pNum + 24));
                }
                // Write configuration data to IRQ enable Register
                logUI.RelOutPortL(cardNumber, registerAddress, writeData, "enable selected IRQs");
            }
        }

        /// <summary>
        /// Function Enables the Index pin for Quadrature counter
        /// </summary>
        /// <param name="isEnable">true to enable Index pin</param>
        private void EnableQdcIndexPin(bool isEnable)
        {            
            UInt32 registerAddress = RegisterOffsetBase.QDC_CONFIG_REG;
            UInt32 writeData = logUI.RelInPortL(cardNumber, registerAddress, "read QDC config register for read/modify/write operation");

            if (isEnable)
                writeData |= RegData.QDC_ENABLE_IDX;        // Set enable Index bit
            else
                writeData &= ~(RegData.QDC_ENABLE_IDX);     // Clear Enable Index bit 

            // Write the updated value to the register
            logUI.RelOutPortL(cardNumber, registerAddress, writeData, "write modified config (index enabled) to QDC reg");
        }

        /// <summary>
        /// Only for Debugging purpose will not include in release version
        /// </summary>
        /// <param name="sender">The object which has raised the event</param>
        /// <param name="e">Additional information about the event</param>
        private void btTestButton_Click(object sender, EventArgs e)
        {
            // Stop the periodic timer 
            PeriodicTimer.Stop();

            TestUIWinform testUI = new TestUIWinform();
            testUI.ShowDialog();

            // Start the periodic timer 
            PeriodicTimer.Start();
        }

        private void button1_Click(object sender, EventArgs e)
        {

        }
    }
}
/***************************************************************************************************************/
/* End of File                                                                                                 */
/***************************************************************************************************************/