﻿/**
  ******************************************************************************
  * @file    McsConfigWinForm.cs
  * @author  Happiest Minds Firmware Team
  * @version v1.0
  * @date    13-December-2016
  * @brief   This file handles the Motor control output configuration dialog box
  ******************************************************************************
  */
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace mPCIe_DIO_24s
{
    public partial class McsConfigWinForm : Form
    {
        /*************************************************************************************************************
         * 
         * Public Functions
         * 
         *************************************************************************************************************/
        /// <summary>
        /// Initialize the McsConfigWinForm components
        /// </summary>
        public McsConfigWinForm()
        {
            InitializeComponent();
        }

        /*************************************************************************************************************
         * 
         * Private Functions
         * 
         *************************************************************************************************************/
        /// <summary>
        /// Initialize the Motor Control Configuration Window
        /// </summary>
        /// <param name="sender">The object which has raised the event</param>
        /// <param name="e">Additional information about the event</param>
        private void McsConfigWinForm_Load(object sender, EventArgs e)
        {
            string[] supportedFrequencyUnit  = new string[] {"Hz", "KHz", "MHz"};
            string[] supportedMotorDirection = new string[] { "Forward", "Reverse" };
            
            // Add the string array to the combo boxes
            cbFreqUnit.Items.AddRange(supportedFrequencyUnit);
            cbMotDir.Items.AddRange(supportedMotorDirection);

            // Set default selected index
            cbFreqUnit.SelectedIndex = 1;   // Default frequency index KHz
            cbMotDir.SelectedIndex = 0;     // Default motor direction forward

            // Load the previous configured values
            Tuple<ePortName, ePinNum> pin;
            pin = ucPinConfig.GetLastUpdatedPortPinName();

            // If already configured, load the previously configured values
            if (Main_Form.ConfigurationParam[(int)pin.Item1, (int)pin.Item2].McsConfig.BaseFreqKhz != 0)
            {
                // Update the previously configured motor direction
                if (Main_Form.ConfigurationParam[(int)pin.Item1, (int)pin.Item2].McsConfig.Direction == eConfigParam.eDirForward)
                    cbMotDir.SelectedIndex = 0;
                else if (Main_Form.ConfigurationParam[(int)pin.Item1, (int)pin.Item2].McsConfig.Direction == eConfigParam.eDirReverse)
                    cbMotDir.SelectedIndex = 1;

                // Load the Frequecy value
                if ((UInt16)Main_Form.ConfigurationParam[(int)pin.Item1, (int)pin.Item2].McsConfig.BaseFreqKhz <= 0)
                {
                    tbBaseFreq.Text = (Main_Form.ConfigurationParam[(int)pin.Item1, (int)pin.Item2].McsConfig.BaseFreqKhz * 1000).ToString();
                    cbFreqUnit.SelectedIndex = 0;
                }
                else
                {
                    tbBaseFreq.Text = Main_Form.ConfigurationParam[(int)pin.Item1, (int)pin.Item2].McsConfig.BaseFreqKhz.ToString();
                    cbFreqUnit.SelectedIndex = 1;
                }

                // Update the previously configured offset count
                tbOffCnt.Text = Main_Form.ConfigurationParam[(int)pin.Item1, (int)pin.Item2].McsConfig.OffsetCount.ToString();
            }
        }

        /// <summary>
        /// Allow only numeric values in the text box. Block all other characters
        /// </summary>
        /// <param name="sender">The object which has raised the event</param>
        /// <param name="e">Additional information about the event</param>
        private void tbBaseFreq_KeyPress(object sender, KeyPressEventArgs e)
        {
            e.Handled = !char.IsDigit(e.KeyChar) && !char.IsControl(e.KeyChar);
        }

        /// <summary>
        /// OK button click event handler. Validate and update the configuration
        /// </summary>
        /// <param name="sender">The object which has raised the event</param>
        /// <param name="e">Additional information about the event</param>
        private void btOk_Click(object sender, EventArgs e)
        {
            eConfigParam direction;
            UInt32 baseFrequency = 0;
            float baseFrequencyKHz = 0;
            UInt32 offsetCount = 0;
                        
            // Validate the entered values
            if (tbBaseFreq.Text == "")
                MessageBox.Show("Please Enter Base Frequency");
            else if (tbOffCnt.Text == "")
                MessageBox.Show("Please Enter Offset Count");
            else
            {               
                // Get the Base Frequency & Offset Count from UI
                try
                {
                    baseFrequency = UInt32.Parse(tbBaseFreq.Text);
                    offsetCount = UInt32.Parse(tbOffCnt.Text);
                }
                catch
                {
                    MessageBox.Show("Entered value is invalid or out of range");
                    return;
                }

                /* Make sure that entered value is non zero */
                if ((baseFrequency == 0) || (offsetCount == 0))
                {
                    MessageBox.Show("Value should be non-zero value");
                    return;
                }

                if (offsetCount > UInt16.MaxValue)
                {
                    MessageBox.Show("Offset count value should be lesser than or equal to " + UInt16.MaxValue);
                    return;
                }

                // Get the selected motor direction value from combo box
                if (cbMotDir.SelectedIndex == 0)
                    direction = eConfigParam.eDirForward;
                else
                    direction = eConfigParam.eDirReverse;

                // Convert the entered frequency value to KHz
                if (cbFreqUnit.SelectedIndex == 0)          // convert from Hz to kHz                
                    baseFrequencyKHz = (float)baseFrequency / 1000;
                else if (cbFreqUnit.SelectedIndex == 1)     // base frequency entered is already in KHz
                    baseFrequencyKHz = (float)baseFrequency;       
                else if (cbFreqUnit.SelectedIndex == 2)     // convert from kHz to MHz                
                    baseFrequencyKHz = (float)baseFrequency * 1000;                
       
                // Update the configurations
                ucPinConfig.MotorControlOutputConfigUpdate(direction, baseFrequencyKHz, offsetCount);

                // Close the pop-up window
                this.Close();
            }
        }

        /// <summary>
        /// MCS Configuration window closed event Handler. Dispose the form
        /// </summary>
        /// <param name="sender">The object which has raised the event</param>
        /// <param name="e">Additional information about the event</param>
        private void McsConfigWinForm_FormClosed(object sender, FormClosedEventArgs e)
        {
            // Dispose of unmanaged resources.
            Dispose(true);
            GC.SuppressFinalize(this);
        }
    }
}
/***************************************************************************************************************/
/* End of File                                                                                                 */
/***************************************************************************************************************/