﻿/**
  ******************************************************************************
  * @file    PulseTrainConfigWinForm.cs
  * @author  Happiest Minds Firmware Team
  * @version v1.0
  * @date    13-December-2016
  * @brief   This file handles the Pulse Train configuration dialog box
  ******************************************************************************
  */
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace mPCIe_DIO_24s
{
    public partial class PulseTrainConfigWinForm : Form
    {
        /*************************************************************************************************************
         * 
         * Public Functions
         * 
         *************************************************************************************************************/
        /// <summary>
        /// Initialize the PulseTrainConfigWinForm components
        /// </summary>
        public PulseTrainConfigWinForm()
        {
            InitializeComponent();
        }

        /*************************************************************************************************************
         * 
         * Private Functions
         * 
         *************************************************************************************************************/
        /// <summary>
        /// Pulse Train Configuration window Load event, Initializes the pulse train Configuration window
        /// </summary>
        /// <param name="sender">The object which has raised the event</param>
        /// <param name="e">Additional information about the event</param>
        private void PulseTrainConfigWinForm_Load(object sender, EventArgs e)
        {
            // Combo box units string
            string[] pulseDirection = new string[] { "Low", "High" };
            string[] timeUnit       = new string[] { "μSec", "mSec", "Sec" };

            // Update the combo boxes 
            cbPulseDir.Items.AddRange(pulseDirection);
            cbPulseDir.SelectedIndex = 1;               // By Default set pulse direction as High Pulse
            cbPulseLenUnit.Items.AddRange(timeUnit);
            cbPulseLenUnit.SelectedIndex = 0;           // By default set pulse length time unit as micro seconds
            cbTimeBwPulUnit.Items.AddRange(timeUnit);
            cbTimeBwPulUnit.SelectedIndex = 0;          // By default set time between pulses time unit as micro seconds

            // Pin is already configured then, Load the previously configured values
            Tuple<ePortName, ePinNum> pin;
            pin = ucPinConfig.GetLastUpdatedPortPinName();

            if (Main_Form.ConfigurationParam[(int)pin.Item1, (int)pin.Item2].PulseTrainConfig.PulseCount != 0)      // Pin is already configured load the values
            {
                // Load the configured pulse direction value
                if (Main_Form.ConfigurationParam[(int)pin.Item1, (int)pin.Item2].PulseTrainConfig.PulseDir == eConfigParam.ePulseDirLow)
                    cbPulseDir.SelectedIndex = 0;
                else
                    cbPulseDir.SelectedIndex = 1;
                
                tbNumOfPulses.Text = Main_Form.ConfigurationParam[(int)pin.Item1, (int)pin.Item2].PulseTrainConfig.PulseCount.ToString();   // Update pulse count value
                tbPulseLength.Text = (Main_Form.ConfigurationParam[(int)pin.Item1, (int)pin.Item2].PulseTrainConfig.PulseLengthMilliSec * 1000).ToString(); // Update Pulse length value
                tbTimeBwPulse.Text = (Main_Form.ConfigurationParam[(int)pin.Item1, (int)pin.Item2].PulseTrainConfig.TimeBetweenPulsesMilliSec * 1000).ToString();   // Update time between pulse value                

                // Check if previous configuration for single pulse generation
                if (Main_Form.ConfigurationParam[(int)pin.Item1, (int)pin.Item2].PulseTrainConfig.TimeBetweenPulsesMilliSec == 0)
                    cbSinglePulse.Checked = true;
            }
        }

        /// <summary>
        /// Text box allows only numeric values, block other characters
        /// </summary>
        /// <param name="sender">The object which has raised the event</param>
        /// <param name="e">Additional information about the event</param>
        private void tbPulseLength_KeyPress(object sender, KeyPressEventArgs e)
        {
            e.Handled = !char.IsDigit(e.KeyChar) && !char.IsControl(e.KeyChar);
        }

        /// <summary>
        /// Text box allows only numeric values, block other characters
        /// </summary>
        /// <param name="sender">The object which has raised the event</param>
        /// <param name="e">Additional information about the event</param>
        private void tbNumOfPulses_KeyPress(object sender, KeyPressEventArgs e)
        {
            e.Handled = !char.IsDigit(e.KeyChar) && !char.IsControl(e.KeyChar);
        }

        /// <summary>
        /// Text box allows only numeric values, block other characters
        /// </summary>
        /// <param name="sender">The object which has raised the event</param>
        /// <param name="e">Additional information about the event</param>
        private void tbTimeBwPulse_KeyPress(object sender, KeyPressEventArgs e)
        {
            e.Handled = !char.IsDigit(e.KeyChar) && !char.IsControl(e.KeyChar);
        }

        /// <summary>
        /// Button OK click event handler
        /// </summary>
        /// <param name="sender">The object which has raised the event</param>
        /// <param name="e">Additional information about the event</param>
        private void btOk_Click(object sender, EventArgs e)
        {
            int pulseLength = 0;
            int timeBwPulse = 0;
            UInt16 pulseCount = 0;
            float pulseLengthMilliSec = 0;
            float timeBwPulseMilliSec = 0;            

            eConfigParam pulseDirection = eConfigParam.eNone;

            //Validate the entered data
            if (tbPulseLength.Text == "")
            {
                MessageBox.Show("Please enter the Pulse Length");
            }
            else if (tbNumOfPulses.Text == "")
            {
                MessageBox.Show("Please enter the Number of pulses");
            }
            else if (tbTimeBwPulse.Text == "")
            {
                MessageBox.Show("Please enter the time between pulse value");
            }
            else
            {
                try
                {
                    // Get Pulse Length, time between pulse & Pulse Count values
                    pulseLength = Int32.Parse(tbPulseLength.Text);
                    timeBwPulse = Int32.Parse(tbTimeBwPulse.Text);
                    pulseCount = UInt16.Parse(tbNumOfPulses.Text);
                }
                catch
                {
                    MessageBox.Show("Entered data is Invalid or out of range");
                    return;
                }

                /* Make sure that entered value is non zero */
                if ((pulseLength == 0) || (pulseCount == 0))
                {
                    MessageBox.Show("Value should be non-zero value");
                    return;
                }

                // Get Pulse Direction (High or Low)
                if (cbPulseDir.SelectedIndex == 0)
                    pulseDirection = eConfigParam.ePulseDirLow;
                else if (cbPulseDir.SelectedIndex == 1)
                    pulseDirection = eConfigParam.ePulseDirHigh;

                /* Convert the Pulse length time to mSec based on the Unit selected */
                if (cbPulseLenUnit.SelectedIndex == 0)           // uSec to uSec
                    pulseLengthMilliSec = (float)pulseLength / 1000;
                else if (cbPulseLenUnit.SelectedIndex == 1)     // Already in mSec                                       
                    pulseLengthMilliSec = pulseLength;
                else if (cbPulseLenUnit.SelectedIndex == 2)     // Sec to mSec
                    pulseLengthMilliSec = pulseLength * 1000;

                /* Convert the time between pulse time value to mSec based on the Unit selected */
                if (cbTimeBwPulUnit.SelectedIndex == 0)                 // uSec to uSec       
                    timeBwPulseMilliSec = (float)timeBwPulse / 1000;
                else if (cbTimeBwPulUnit.SelectedIndex == 1)            // Already in mSec       
                    timeBwPulseMilliSec = timeBwPulse;
                else if (cbTimeBwPulUnit.SelectedIndex == 2)            // Sec to mSec                  
                    timeBwPulseMilliSec = timeBwPulse * 1000;

                // Update the Pulse Train configuration 
                ucPinConfig.PulseTrainConfigUpdate(pulseDirection, pulseLengthMilliSec, pulseCount, timeBwPulseMilliSec);
                
                // Close the pop-up configuration window
                this.Close();
            }
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="sender">The object which has raised the event</param>
        /// <param name="e">Additional information about the event</param>
        private void cbSinglePulse_CheckedChanged(object sender, EventArgs e)
        {
            tbTimeBwPulse.Text = 0.ToString();

            if (cbSinglePulse.Checked == true)
            {
                // Selected for single pulse, disable Time between pulse Number of pulses text box
                tbTimeBwPulse.Enabled = false;
                cbTimeBwPulUnit.Enabled = false;                
                tbNumOfPulses.Enabled = false;
                // Set the Number of pulses as 1 for single pulse
                tbNumOfPulses.Text = "1";
            }                
            else
            {
                // Enable Time between pulses & Pulse count text boxes
                tbTimeBwPulse.Enabled = true;
                cbTimeBwPulUnit.Enabled = true;
                tbNumOfPulses.Enabled = true;
            }                
        }

        /// <summary>
        /// Pulse Train Configuration window closed event Handler. Dispose the unmanaged resources.
        /// </summary>
        /// <param name="sender">The object which has raised the event</param>
        /// <param name="e">Additional information about the event</param>
        private void PulseTrainConfigWinForm_FormClosed(object sender, FormClosedEventArgs e)
        {
            // Dispose of unmanaged resources.
            Dispose(true);
            GC.SuppressFinalize(this);
        }
    }
}
/***************************************************************************************************************/
/* End of File                                                                                                 */
/***************************************************************************************************************/