﻿/**
  ********************************************************************************************************************
  * @file    ucPinConfig.cs
  * @author  Happiest Minds Firmware Team
  * @version v1.0
  * @date    14-December-2016
  * @brief   This source file initializes the User control & used to get and set data from/to user control
  ********************************************************************************************************************
  */
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace mPCIe_DIO_24s
{
    public partial class ucPinConfig : UserControl
    {
        /*************************************************************************************************************
         * 
         * Private Constant declarations
         * 
         *************************************************************************************************************/
        // Supported Input string
        private const string GENERAL_OUTPUT         = "General Output";
        private const string PULSE_TRAIN_OUTPUT     = "Generate Pulse Train";
        private const string PWM_OUTPUT             = "Generate PWM";
        private const string MCS_OUTPUT             = "Motor Control Output";
        // Supported Output string
        private const string GENERAL_INPUT          = "General Input";
        private const string DETECT_PULSE_INPUT     = "Detect Pulse (Event)";
        private const string EVENT_COUNTER_INPUT    = "Event Counter";
        private const string QDC_INPUT              = "Quadrature Pulse Counter";
        private const string MEASURE_PULSE_WIDTH    = "Measure Pulse Width";
        private const string MEASURE_FREQUENCY      = "Measure Frequency";
        private const string MEASURE_DUTY_CYCLE     = "Measure Duty Cycle";
        private const string DETECT_LTOH_EVENT      = "Detect L-H Event";

        /*************************************************************************************************************
         * 
         * Private Variable declarations
         * 
         *************************************************************************************************************/        
        private static string lastUpdatedPinName = null;        // Variable holds the last updated port & pin details
        private static int comboboxAltFunPreviousValue = 0;     // Variable holds the previous selection of alternate combobox position
        private int alternateFunction;                          // Last Selected alternate function
        private char m_portName;                                // Port name in the form (A, B or C)
        private int m_pinNumber;                                // Pin Number from (0-7)

        /*************************************************************************************************************
         * 
         * Public Functions
         * 
         *************************************************************************************************************/
        /// <summary>
        /// Set or get the port name property
        /// </summary>
        public int PortName
        {
            get
            {
                return m_portName;
            }
            set
            {
                m_portName = (char)value;                
                lbPinName.Text = "P" + m_portName + "[" + m_pinNumber + "]";
            }
        }

        /// <summary>
        /// Set or get the Pin Number property
        /// </summary>
        public int PinNumber
        {
            get
            {
                return m_pinNumber;
            }
            set
            {
                m_pinNumber = value;
                lbPinName.Text = "P"+ m_portName + "[" + m_pinNumber + "]";
            }
        }

        /// <summary>
        /// Set or get the Alternate Function type
        /// </summary>
        public int AltFunction
        {
            get
            {
                return alternateFunction;
            }
            set
            {
                alternateFunction = value;                
            }            
        }        

        /// <summary>
        /// Initialize the uiPinConfig controls
        /// </summary>
        public ucPinConfig()
        {
            InitializeComponent();                        
        }        

        /// <summary>
        /// Returns port name and pin from the port name string 
        /// </summary>
        /// <returns>Port Name & Pin Number</returns>
        public static Tuple<ePortName, ePinNum> GetLastUpdatedPortPinName()
        {
            ePortName portName = ePortName.ePortUnknown;
            ePinNum pinName = ePinNum.ePinUnknown ;

            // Walkthrough all pins to get the last updated pin match
            for (int i = 0; i < 8; i++)
            {
                if (lastUpdatedPinName == ("A" + i))
                {                    
                    // last updated from port A and pin i
                    portName = ePortName.ePortA;
                    pinName = (ePinNum)i;
                    break;
                }
                if (lastUpdatedPinName == ("B" + i))
                {
                    // last updated from port B and pin i
                    portName = ePortName.ePortB;
                    pinName = (ePinNum)i;
                    break;
                }
                if (lastUpdatedPinName == ("C" + i))
                {
                    if (i <= 3)
                    {
                        // last updated from port CL and pin i
                        portName = ePortName.ePortCL;
                        pinName = (ePinNum)i;
                    }
                    else
                    {
                        // last updated from port CH and pin i
                        portName = ePortName.ePortCH;
                        pinName = (ePinNum)i;
                    }
                    break;
                }
            }
            return new Tuple<ePortName, ePinNum>(portName, pinName);
        }

        /// <summary>
        /// Pulse detect configuration update callback
        /// </summary>
        /// <param name="triggerType">trigger type; rising/falling/low pulse or high pulse</param>
        /// <param name="isIrqEnable">true to enable the IRQ</param>
        public void PulseDetectConfigUpdate(eConfigParam triggerType, bool isIrqEnable)
        {
            UpdateConfigParam configParam = new UpdateConfigParam();

            Tuple<ePortName, ePinNum> pin;

            // Get Port name and pin number
            pin = GetLastUpdatedPortPinName();

            // Update the Pulse Detect configuration to updateConfigParam Structure
            configParam.PortName = (ePortName)pin.Item1;
            configParam.PinNumber = (ePinNum)pin.Item2;
            configParam.AltFun = eAltFunction.ePulseDetect;
            configParam.PulseDetectConfig.TriggerType = triggerType;
            configParam.PulseDetectConfig.EnableIrq = isIrqEnable;

            // Update the configuration parameters to registers
            Main_Form.UpdatePinConfig(configParam);
        }

        /// <summary>
        /// Function updates the Pulse Train configuration
        /// </summary>
        /// <param name="pulseDirection">specifies the pulse direction high or low</param>
        /// <param name="pulseLengthMilliSec">Pulse length in mSec</param>
        /// <param name="pulseCount">Number of pulses</param>
        /// <param name="timeBwPulsesMilliSec">Time between pulses in mSec</param>
        public static void PulseTrainConfigUpdate (eConfigParam pulseDirection, float pulseLengthMilliSec, UInt16 pulseCount, float timeBwPulsesMilliSec)
        {
            UpdateConfigParam configParam = new UpdateConfigParam();
            Tuple<ePortName, ePinNum> pin;

            // Get Port name and pin number
            pin = GetLastUpdatedPortPinName();

            // Update the Pulse train configuration to updateConfigParam Structure
            configParam.PortName = (ePortName)pin.Item1;
            configParam.PinNumber = (ePinNum)pin.Item2;
            configParam.AltFun = eAltFunction.eGenPulseTrain;
            configParam.PulseTrainConfig.PulseDir = pulseDirection;
            configParam.PulseTrainConfig.PulseLengthMilliSec = pulseLengthMilliSec;
            configParam.PulseTrainConfig.PulseCount = pulseCount;            
            configParam.PulseTrainConfig.TimeBetweenPulsesMilliSec = timeBwPulsesMilliSec;

            // Update the configuration parameters to registers
            Main_Form.UpdatePinConfig(configParam);
        }

        /// <summary>
        /// Function updates the Duty cycle configuration
        /// </summary>
        /// <param name="timePeriodMilliSec">Total time period of the signal in mSec</param>
        /// <param name="dutyCycle">Duty cycle of the signal in %</param>
        public static void DutyCycleConfigUpdate(float timePeriodMilliSec, int dutyCycle)
        {
            UpdateConfigParam configParam = new UpdateConfigParam();
            Tuple<ePortName, ePinNum> pin;

            // Get Port name and pin number
            pin = GetLastUpdatedPortPinName();

            configParam.PortName = (ePortName)pin.Item1;
            configParam.PinNumber = (ePinNum)pin.Item2;
            configParam.AltFun = eAltFunction.eGenDutyCycle;
            configParam.DutyCycleConfig.TimePeriodMilliSec = timePeriodMilliSec;
            configParam.DutyCycleConfig.DutyCycle = dutyCycle;

            // Update the configuration parameters to registers
            Main_Form.UpdatePinConfig(configParam);
        }

        /// <summary>
        /// Function updates the Quadrature Input Configurations
        /// </summary>
        /// <param name="resetSource"> Quadrature reset source (Pin Name)</param>
        /// <param name="resetType"> Quadrature reset type </param>
        public static void QuadratureControlInputConfigUpdate(string resetSource)
        {
            eStatus result;
            UpdateConfigParam configParam = new UpdateConfigParam();
            Tuple<ePortName, ePinNum> pin;

            // Get Port name and pin number           
            pin = GetLastUpdatedPortPinName();

            // Update the Pin name & Number, alternate function to the updateConfigParam structure
            configParam.PortName = (ePortName)pin.Item1;
            configParam.PinNumber = (ePinNum)pin.Item2;
            configParam.AltFun = eAltFunction.eCntQuadPulse;

            // Get the reset Source
            resetSource = resetSource.Remove(0, 1);
            lastUpdatedPinName = resetSource;

            // Get Port name and pin number
            pin = GetLastUpdatedPortPinName();

            // Update the updateConfigParam structure
            configParam.QdcConfig.IsResetConfig = true;
            configParam.QdcConfig.QdcResetPinName = pin.Item1;
            configParam.QdcConfig.QdcResetPinNumber = pin.Item2;            

            // Update the configuration parameters to registers
            result = Main_Form.UpdatePinConfig(configParam);

            // Check if configuresation is successful
            if (result != eStatus.eSuccess)
                Main_Form.UncheckResetOnEvent();       // Configuration failed, uncheck the reset on event check box         
        }

        /// <summary>
        /// Function updates the Quadrature Output Configurations
        /// </summary>
        /// <param name="direction">Motor control direction (Forward or reverse)</param>
        /// <param name="baseFreqKhz">Base frequency in KHz</param>
        /// <param name="offsetCount">Offset count specifies the number of steps</param>
        public static void MotorControlOutputConfigUpdate(eConfigParam direction, float baseFreqKhz, UInt32 offsetCount)          
        {            
            UpdateConfigParam configParam = new UpdateConfigParam();
            Tuple<ePortName, ePinNum> pin;

            // Get Port name and pin number
            pin = GetLastUpdatedPortPinName();

            // Update the configuration parameters to updateConfigParam structure
            configParam.PortName = (ePortName)pin.Item1;
            configParam.PinNumber = (ePinNum)pin.Item2;  // Pin is pointing to (actual pin + 1) because, UI updates the 2 quadrature lines
            configParam.AltFun = eAltFunction.eGenQuad;
            configParam.McsConfig.Direction = direction;
            configParam.McsConfig.BaseFreqKhz = baseFreqKhz;
            configParam.McsConfig.OffsetCount = offsetCount;

            // Update the configuration parameters to registers
            Main_Form.UpdatePinConfig(configParam);
        }

        /// <summary>
        /// Function Updates the Event Counter Configuration
        /// </summary>
        /// <param name="countType">Count type (either on rising/falling edge or high/low pulse)</param>
        /// <param name="countThreshold">Counter threshold value (0-255 max)</param>
        public static void EventCounterConfigUpdate(eConfigParam countType, UInt16 countThreshold)
        {
            UpdateConfigParam configParam = new UpdateConfigParam();
            Tuple<ePortName, ePinNum> pin;

            // Get Port name and pin number
            pin = GetLastUpdatedPortPinName();

            // Update the configuration parameters to updateConfigParam structure
            configParam.PortName = (ePortName)pin.Item1;
            configParam.PinNumber = (ePinNum)pin.Item2;  // Pin is pointing to (actual pin + 1) because, UI updates the 2 quadrature lines
            configParam.AltFun = eAltFunction.eCntPulse;
            configParam.EventCounterConfig.TriggerType = countType;
            configParam.EventCounterConfig.CounterThreshold = countThreshold;

            // Update the configuration parameters to registers
            Main_Form.UpdatePinConfig(configParam);
        }

        /// <summary>
        /// Updates the Alternate function combo box based on the selected direction
        /// </summary>
        /// <param name="direction">Port direction (Input or Output)</param>
        /// <param name="portName">Port Name</param>
        /// <param name="pinNum">Pin Number</param>
        public void UpdatePinAlternateFunction(eDirection direction, int portName, int pinNum)
        {
            string[] inputAlternateFunctions = null;
            string[] outputAlternateFunctions = null;
         
            if (portName == 'A')   // Port A
            {
                // Port A's supported feature
                inputAlternateFunctions = new string[] { GENERAL_INPUT, DETECT_PULSE_INPUT, EVENT_COUNTER_INPUT, MEASURE_PULSE_WIDTH, MEASURE_FREQUENCY, MEASURE_DUTY_CYCLE };
                outputAlternateFunctions = new string[] { GENERAL_OUTPUT };
            }
            else if (portName == 'B')  // Port B
            {
                // Port B's supported feature
                inputAlternateFunctions = new string[] { GENERAL_INPUT };
                outputAlternateFunctions = new string[] { GENERAL_OUTPUT, PULSE_TRAIN_OUTPUT, PWM_OUTPUT };                    
            }
            else if (portName == 'C')  // Port C
            {
                if (pinNum < 4) // Port CL
                {
                    if (pinNum == 0)
                        inputAlternateFunctions = new string[] { GENERAL_INPUT, DETECT_LTOH_EVENT, QDC_INPUT };
                    else
                        inputAlternateFunctions = new string[] { GENERAL_INPUT, DETECT_LTOH_EVENT };

                    outputAlternateFunctions = new string[] { GENERAL_OUTPUT, PULSE_TRAIN_OUTPUT };
                }    
                else        // Port CH
                {
                    if (pinNum == 4)
                        outputAlternateFunctions = new string[] { GENERAL_OUTPUT, PULSE_TRAIN_OUTPUT, MCS_OUTPUT };
                    else
                        outputAlternateFunctions = new string[] { GENERAL_OUTPUT, PULSE_TRAIN_OUTPUT };


                    inputAlternateFunctions = new string[] { GENERAL_INPUT, DETECT_LTOH_EVENT };                        
                }                
            }                

            // Add the strings to the combo box
            if (direction == eDirection.eInput)
            {
                cbAltFun.Items.Clear();                             // Clear the existing entries
                cbAltFun.Items.AddRange(inputAlternateFunctions);   // Add the Input alternate functions
                cbAltFun.SelectedIndex = 0;                         // Make General Input as default alternate function
            }
            else if (direction == eDirection.eOutput)
            {
                cbAltFun.Items.Clear();                             // Clear the existing entries
                cbAltFun.Items.AddRange(outputAlternateFunctions);  // Add the Output alternate functions
                cbAltFun.SelectedIndex = 0;                         // Make General Output as default alternate function
            }
        }

        /// <summary>
        /// Function Updates the User control for Quadrature Input A & B
        /// </summary>
        /// <param name="userControl">User control for the particular port and pin</param>
        /// <param name="lineNumber">Line number species the QDC/MCS line 1 or 2</param>
        /// <param name="afFunction">Alternate function specifies the QDC or MCS</param>
        public static void UpdateQuadratureUI(ucPinConfig userControl, int lineNumber, eAltFunction afFunction)
        {
            // Clear the existing user control panel
            userControl.ClearPanel();
            if (afFunction == eAltFunction.eCntQuadPulse)       /* Quadrature Input configuration update */
            {
                if (lineNumber == 1)    // QDC Line-A
                {
                    // Make Reset on event combo box as visible
                    userControl.panResetOnEvent.Visible = true;
                }
                else                    //QDC - Line-B
                {                    
                    userControl.cbAltFun.Enabled = false;       // Disable the alternate function select and clear the content
                    userControl.cbAltFun.Items.Clear();
                    userControl.panMotOp.Visible = true;
                    userControl.btMotorOpStart.Text = "Reset";  // Make the button name as Reset
                    
                }
            }
            else if (afFunction == eAltFunction.eGenQuad)       // Motor Control Output Configuration
            {
                if (lineNumber == 1)        // MCS Line-A
                {
                    userControl.panMotOp.Visible = true;
                    userControl.btMotorOpStart.Text = "Edit";   // Make the button name as Edit                      
                }
                else                        // MCS Line-B
                {                             
                    userControl.cbAltFun.Enabled = false;       // Disable the alternate function select and clear the content
                    userControl.cbAltFun.Items.Clear();

                    userControl.panMotOp.Visible = true;                    
                    userControl.tbMotOpStatus.Visible = false;
                    userControl.btMotorOpStart.Text = "Start";  // Make the button name as Start
                }
            }
        }

        /// <summary>
        /// Updates the user interface based on the input parameter
        /// </summary>
        /// <param name="param">parameter which needs to updated</param>
        /// <param name="data">data</param>
        public void UpdateInputValue(eConfigParam param, double data)
        {
            switch (param)
            {
                case eConfigParam.eGenInput:
                    if (data == 0)
                        tbData.Text = "Low";
                    else if (data == 1)
                        tbData.Text = "High";

                    break;

                case eConfigParam.eRisingEdge:
                    tbMotOpStatus.Text = "Rising Edge Detected";
                    break;

                case eConfigParam.eFallingEdge:
                    tbMotOpStatus.Text = "Falling Edge Detected";
                    break;

                case eConfigParam.eHighPulse:
                    tbMotOpStatus.Text = "High Pulse Detected";
                    break;

                case eConfigParam.eLowPulse:
                    tbMotOpStatus.Text = "Low Pulse Detected";
                    break;

                default:
                    break;
            }
        }

        /// <summary>
        /// Function enable/Disables the QDC's reset pin 
        /// </summary>
        /// <param name="userControl">Control to the Reset pin</param>
        /// <param name="isDisable">Value true will disable the Reset source pin</param>
        public void UpdateQdcResetPin(ucPinConfig userControl, bool isDisable)
        {
            if (isDisable)
            {
                userControl.Enabled = false;       // Disables the QDC's reset pin 
            }
            else
            {
                userControl.Enabled = true;         // Enable the QDC's reset pin 
                cbAltFun.SelectedIndex = 0;         // Make the default alternate function as general input
            }
        }

        /// <summary>
        /// Function updates the configured parameter on the UI
        /// </summary>        
        /// <param name="message">Message to be displayed on the text box</param>
        public void UpdateDisplayConfig(string message)
        {
            tbMotOpStatus.Text = message;            
        }

        /// <summary>
        /// Function updates the General input state on the Display
        /// </summary>        
        /// <param name="message">Message to be displayed on the text box</param>
        public void UpdateGenInState(string message)
        {
            tbData.Text = message;
        }

        /// <summary>
        /// Function updates the Measured Frequency
        /// </summary>        
        /// <param name="message">Message to be displayed on the text box</param>
        public void UpdateFrequency(string message)
        {
            tbFreq.Text = message;
        }

        /// <summary>
        /// Function updates the Measured Duty cycle
        /// </summary>        
        /// <param name="message"></param>
        public void UpdateDutyCycleValue(string message)
        {
            tbFreq.Text = message + " %";            
        }

        /// <summary>
        /// Function updates the Measure Pulse Width
        /// </summary>        
        /// <param name="message1">Message to be displayed on High Pulse duration</param>
        /// <param name="message2">Message to be displayed on the Low Pulse duration</param>
        public void UpdatePulseHighMeasure(string message1, string message2)
        {
            tbPWHigh.Text = message1;
            tbPWLow.Text = message2;            
        }

        /// <summary>
        /// Function check or uncheck the reset on event combo box
        /// </summary>        
        /// <param name="isChecked">True to check the combobox</param>
        public void UpdateResetOnEventCheckBox(bool isChecked)
        {
            cbResetOnEvent.Checked = isChecked;
        }

        /*************************************************************************************************************
         * 
         * Private Functions
         * 
         *************************************************************************************************************/       
        /// <summary>
        /// Event handler for Alternate function selection combo box index changed
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void cbAltFun_SelectedIndexChanged(object sender, EventArgs e)
        {         
            ComboBox comboBoxControl = (ComboBox)sender;            
            Tuple<ePortName, ePinNum> pin;
            UpdateConfigParam configParam = new UpdateConfigParam();
            string newPinName;
            DialogResult dialogResult;

            // Get the selected alternate function string
            string selectedAlternateFunction = comboBoxControl.Text;

            /* Get Port Number and Pin Number */
            lastUpdatedPinName = comboBoxControl.Parent.Name;

            // Get Port name and pin number
            pin = GetLastUpdatedPortPinName();

            /* Update the Port Name and Pin Number */
            configParam.PortName = pin.Item1;
            configParam.PinNumber = pin.Item2;

            // Enable all the alternate function combo boxes
            cbAltFun.Enabled = true;

            // Clear the previous configuration display message
            tbMotOpStatus.Text = "";

            switch (selectedAlternateFunction)
            {
                /*------------------------------- INPUT -------------------------------*/
                case GENERAL_INPUT:
                    // Clear existing panel and display General input panel
                    ClearPanel();
                    panGenInput.Visible = true;
                    tbData.Text = "--";

                    if (configParam.PortName == ePortName.ePortB)  // Debounce Logic is not supported for Port B
                        chBxDebounce.Enabled = false;

                    configParam.AltFun = eAltFunction.eGenIn;
                    configParam.EnableDebounce = chBxDebounce.Checked;

                    // Update the configuration parameters to registers
                    Main_Form.UpdatePinConfig(configParam);
                    break;

                case DETECT_PULSE_INPUT:
                    // Clear existing panel and display new panel
                    ClearPanel();
                    panMotOp.Visible = true;
                    btMotorOpStart.Text = "Clear";
                    // display the pulse detect configuration dialog box
                    DetectPulseConfigWinForm wf = new DetectPulseConfigWinForm();
                    wf.ShowDialog();
                    break;

                case DETECT_LTOH_EVENT:
                    // Clear existing panel and display new panel
                    ClearPanel();
                    panMotOp.Visible = true;
                    btMotorOpStart.Text = "Clear";                    
                    configParam.AltFun = eAltFunction.eLtoHDetect;

                    // Update the configuration parameters to registers
                    Main_Form.UpdatePinConfig(configParam);
                    break;

                case EVENT_COUNTER_INPUT:
                    // Clear existing panel and display new panel
                    ClearPanel();
                    panFreqMeasure.Visible = true;                                 

                    // Display Pulse Detect configuration dialog
                    EventCntConfigWinform eventCounterCofnigDialogCtrl = new EventCntConfigWinform();
                    eventCounterCofnigDialogCtrl.ShowDialog();                    
                    break;

                case MEASURE_PULSE_WIDTH:
                    // Clear existing panel and display Pulse measure panel
                    ClearPanel();
                    panPWMeasure.Visible = true;
                    configParam.AltFun = eAltFunction.eMeasPulseHigh;

                    // Update the configuration parameters to registers
                    Main_Form.UpdatePinConfig(configParam);
                    break;

                case MEASURE_FREQUENCY:
                    // Clear existing panel and display Frequency measure panel
                    ClearPanel();

                    // Update the configuration parameters
                    panFreqMeasure.Visible = true;
                    configParam.AltFun = eAltFunction.eMeasFreq;

                    // Update the configuration parameters to registers
                    Main_Form.UpdatePinConfig(configParam);
                    break;

                case MEASURE_DUTY_CYCLE:
                    // Clear existing panel and display Measure Duty cycle panel
                    ClearPanel();

                    // Update the configuration parameters
                    panFreqMeasure.Visible = true;
                    configParam.AltFun = eAltFunction.eMeasDutyCycle;

                    // Update the configuration parameters to registers
                    Main_Form.UpdatePinConfig(configParam);
                    break;

                case QDC_INPUT:
                    // Check if Quadrature counter is available. Note: At any point of time, mPCIe 24s supports only pair of quadrature counter 
                    if (!Main_Form.IsQdcUsed)
                    {
                        // QDC is available proceed with the configuration
                        newPinName = lastUpdatedPinName;
                        newPinName = newPinName.Remove(1) + ((UInt16)configParam.PinNumber + 1).ToString();  // Get the QDC-B pin name
                        dialogResult = MessageBox.Show("Quadrature input is a pair of signals. \r\nLines P" + lastUpdatedPinName + " & P" + newPinName + " will be configured as Quadrature input." +
                                                                    " \r\rDo you want to continue?", "Warning", MessageBoxButtons.YesNo);
                        if (dialogResult == DialogResult.Yes)
                        {                       
                            // Get the Pin Number                                 
                            pin = GetLastUpdatedPortPinName();

                            // Display new panel
                            Main_Form.updateQuadratureInputUI(pin.Item1, pin.Item2);

                            // Set the alternate function as QDC
                            configParam.AltFun = eAltFunction.eCntQuadPulse;

                            // Update the configuration parameters to registers
                            Main_Form.UpdatePinConfig(configParam);

                            // Disable the Combo box
                            // NOTE: QDC are pair of signals user is not allowed to change the pin’s alternate function, hence it is disabled. It can be re-enabled by selecting the direction
                            cbAltFun.Enabled = false;                            
                        }
                        else
                        {
                            // Revert back the alternate function to the previous position
                            cbAltFun.SelectedIndex = comboboxAltFunPreviousValue;
                        }
                    }
                    else
                    {
                        // QDC is not available Display error message
                        MessageBox.Show("Only one Quadrature Counter is allowed");

                        // Revert back the alternate function to the previous position
                        cbAltFun.SelectedIndex = comboboxAltFunPreviousValue;
                    }
                    break;
                
                /*------------------------------- OUTPUT -------------------------------*/
                case GENERAL_OUTPUT:
                    // Clear existing panel and display the general output panel
                    ClearPanel();
                    rbLow.Checked = true;               // By default set all the output to low state
                    panGenOutput.Visible = true;

                    // Set the alternate function as general output
                    configParam.AltFun = eAltFunction.eGenOut;

                    // Update the configuration parameters to registers
                    Main_Form.UpdatePinConfig(configParam);    
                    break;

                case PULSE_TRAIN_OUTPUT:
                    // Clear existing panel and display the new panel
                    ClearPanel();
                    panMotOp.Visible = true;
                    btMotorOpStart.Text = "Start";

                    // Display Pulse Train generation Configuration dialog
                    PulseTrainConfigWinForm pulseTrainConfig = new PulseTrainConfigWinForm();                    
                    pulseTrainConfig.ShowDialog();                        
                    break;

                case PWM_OUTPUT:
                    // Clear existing panel and display the new panel
                    ClearPanel();
                    panMotOp.Visible = true;
                    btMotorOpStart.Text = "Start";

                    // Display PWM Configuration window
                    PWMGenConfigWinform pwmConfig = new PWMGenConfigWinform();
                    pwmConfig.ShowDialog();                    
                    break;

                case MCS_OUTPUT:
                    // Check if Quadrature counter is available. Note: At any point of time, mPCIe 24s supports only pair of Motor Control Output 
                    if (!(Main_Form.IsMcsUsed))
                    {           
                        // Get the MCS-B Pin name                        
                        newPinName = lastUpdatedPinName;
                        newPinName = newPinName.Remove(1) + ((UInt16)configParam.PinNumber + 1).ToString();
                        dialogResult = MessageBox.Show("Quadrature output is a pair of signals. \r\nLines P" + lastUpdatedPinName + " & P" + newPinName + " will be configured as Quadrature output." +
                                                                    " \r\rDo you want to continue?", "Warning", MessageBoxButtons.YesNo);

                        if (dialogResult == DialogResult.Yes)
                        {
                            // Display the new panel
                            pin = GetLastUpdatedPortPinName();
                            Main_Form.updateQuadratureOutputUI(pin.Item1, pin.Item2);

                            // Display Motor Control Configuration window
                            McsConfigWinForm mcsConfigDialogCtrl = new McsConfigWinForm();
                            mcsConfigDialogCtrl.ShowDialog();

                            configParam.PinNumber = pin.Item2 + 1;
                            // Update the configuration parameters to registers
                            Main_Form.UpdatePinConfig(configParam);
                            cbAltFun.Enabled = false;
                        }
                        else
                        {
                            // Revert back the alternate function to the previous position
                            cbAltFun.SelectedIndex = comboboxAltFunPreviousValue;
                        }
                    }
                    else
                    {
                        MessageBox.Show("Only one Motor control Output is allowed");
                        // Revert back the alternate function to the previous position
                        cbAltFun.SelectedIndex = comboboxAltFunPreviousValue;
                    }                    
                    break;

                default:
                    // Unknown selection, should not come here
                    fLytPanAltFun.Controls.Clear();
                    break;
            }
        }

        /// <summary>
        /// Event handler for General output configuration
        /// </summary>
        /// <param name="sender">The object which has raised the event</param>
        /// <param name="e">Additional information about the event</param>
        private void rbHigh_CheckedChanged(object sender, EventArgs e)
        {            
            // Disable the event, this is to avoid multiple trigger of the events
            this.rbHigh.CheckedChanged -= new System.EventHandler(this.rbHigh_CheckedChanged);

            // General Output has changed the state
            lastUpdatedPinName = fLytPanAltFun.Parent.Name;
            Tuple<ePortName, ePinNum> pin;

            // Get Port name and pin number
            pin = GetLastUpdatedPortPinName();          

            if (rbHigh.Checked == true)
            {
                // Make the line High
                if (Main_Form.UpdateOutputState(pin.Item1, pin.Item2, true) != eStatus.eSuccess)
                {
                    // Make the line High failed due to unknown reason. Revert back the UI to indicate Low
                    rbHigh.Checked = false;
                    rbLow.Checked = true;
                }
            }
            else
            {
                if (Main_Form.UpdateOutputState(pin.Item1, pin.Item2, false) != eStatus.eSuccess)
                {
                    // Make the line Low failed due to unknown reason. Revert back the UI to indicate High
                    rbLow.Checked = false;
                    rbHigh.Checked = true;                    
                }
            }
            /* Re-enable the event */
            this.rbHigh.CheckedChanged += new System.EventHandler(this.rbHigh_CheckedChanged);
        }

        /// <summary>
        /// Event handler for motor o/p start click
        /// </summary>
        /// <param name="sender">The object which has raised the event</param>
        /// <param name="e">Additional information about the event</param>
        private void btMotorOpStart_Click(object sender, EventArgs e)
        {
            UpdateConfigParam configParam = new UpdateConfigParam();
            lastUpdatedPinName = fLytPanAltFun.Parent.Name;

            Tuple<ePortName, ePinNum> pin;

            // Get Port name and pin number
            pin = GetLastUpdatedPortPinName();
            configParam.PortName = pin.Item1;
            configParam.PinNumber = pin.Item2;

            if (btMotorOpStart.Text == "Clear")
            {
                // Alternate function will be "Pulse detect (Event)"
                tbMotOpStatus.Clear();  // Clear the text box
            }
            else if ((btMotorOpStart.Text == "Reset") && (cbAltFun.Text == ""))
            {
                // Clear the QDC counter
                tbMotOpStatus.Text = "0";
                configParam.AltFun = eAltFunction.eCntQuadPulse;
                configParam.ResetCounter = true;

                // Update the configuration parameters to registers
                Main_Form.UpdatePinConfig(configParam);
            }
            else if (btMotorOpStart.Text == "Start")
            {
                // Set the MCS control bit
                if (cbAltFun.Text == "")
                    Main_Form.SetMCSCtrlBit(configParam.PortName, (configParam.PinNumber - 1));
                else
                    Main_Form.SetOutputCtrlBit(configParam.PortName, configParam.PinNumber);    // Set the Output control bit
            }
            else if (btMotorOpStart.Text == "Edit")
            {
                // Display MCS Configuration window
                McsConfigWinForm mcsConfigDialogCtrl = new McsConfigWinForm();
                mcsConfigDialogCtrl.ShowDialog();
            }
        }

        /// <summary>
        /// Debounce Enable/Disable UI event handler
        /// </summary>
        /// <param name="sender">The object which has raised the event</param>
        /// <param name="e">Additional information about the event</param>
        private void chBxDebounce_CheckedChanged(object sender, EventArgs e)
        {
            UpdateConfigParam configParam = new UpdateConfigParam();
            Tuple<ePortName, ePinNum> pin;
            lastUpdatedPinName = fLytPanAltFun.Parent.Name;

            // Get Port name and pin number
            pin = GetLastUpdatedPortPinName();

            // Update the debounce configuration to the updateConfigParam structure
            configParam.AltFun = eAltFunction.eGenIn;
            configParam.PortName = pin.Item1;
            configParam.PinNumber = pin.Item2;
            configParam.EnableDebounce = chBxDebounce.Checked;

            // Update the configuration parameters to registers
            Main_Form.UpdatePinConfig(configParam);
        }

        /// <summary>
        /// Event handler for Quadrature input reset configuration changed
        /// </summary>
        /// <param name="sender">The object which has raised the event</param>
        /// <param name="e">Additional information about the event</param>
        private void cbResetOnEvent_CheckedChanged(object sender, EventArgs e)
        {
            UpdateConfigParam configParam = new UpdateConfigParam();
            Tuple<ePortName, ePinNum> pin;
            DialogResult dialogResult;
            string indexPinName = null;            

            if (cbResetOnEvent.Checked == true)
            {
                // Get the QDC pin name & number
                pin = Main_Form.GetQdcPortPinDetails();
                ePortName qdcPortName = pin.Item1;
                ePinNum qdcPinNumber = pin.Item2;

                // Check which pin will be Quadrature Counter's Index pin
                if (Main_Form.ConfigurationParam[(int)qdcPortName, (int)qdcPinNumber].PinNumber == ePinNum.ePin0)
                    indexPinName = "PC2";     // For PCL0 & PCL1 quadrature inputs PC2 will be the Index pin
                else
                    indexPinName = "PC1";     // For PCL2 & PCL3 quadrature inputs PC1 will be the Index pin

                // Display Dialog conformation box
                dialogResult = MessageBox.Show("Do you want to enable " + indexPinName + " as reset source for Quadrature Counter", "Warning", MessageBoxButtons.YesNo);
                if (dialogResult == DialogResult.Yes)                                    
                    ucPinConfig.QuadratureControlInputConfigUpdate(indexPinName);   // Update the configuration                                 
                else                                    
                    cbResetOnEvent.Checked = false;     // User rejected the request, uncheck the reset on event checkbox
            }
            else
            {
                // Get Port name and pin number
                pin = GetLastUpdatedPortPinName();

                // Update the configuration parameters
                configParam.PortName = pin.Item1;
                configParam.PinNumber = pin.Item2;
                configParam.AltFun = eAltFunction.eCntQuadPulse;
                configParam.ResetCounter = false;
                configParam.QdcConfig.IsResetConfig = true;
                configParam.QdcConfig.QdcResetTriggerType = eConfigParam.eNone;

                // Update the configuration parameters to registers
                Main_Form.UpdatePinConfig(configParam);
            }
        }

        /// <summary>
        /// Event handler for click on cbAltFun combobox, Function saves the current selected index
        /// </summary>
        /// <param name="sender">The object which has raised the event</param>
        /// <param name="e">Additional information about the event</param>
        private void cbAltFun_Click(object sender, EventArgs e)
        {
            // Save the Current Selected index to local variable
            // This will be used to revert back the configuration in case of configuration failure
            comboboxAltFunPreviousValue = cbAltFun.SelectedIndex;            
        }

        /// <summary>
        /// Clears the configure/monitor space to update new control
        /// </summary>
        private void ClearPanel()
        {
            panGenOutput.Visible = false;
            panPWMeasure.Visible = false;
            panMotOp.Visible = false;
            panFreqMeasure.Visible = false;
            panGenInput.Visible = false;
            panResetOnEvent.Visible = false;
            tbMotOpStatus.Visible = false;
            tbMotOpStatus.Visible = true;   // make the tbMotOpStatus text box to default state visible 
        }
    }
}
/***************************************************************************************************************/
/* End of File                                                                                                 */
/***************************************************************************************************************/